CREATE OR REPLACE PACKAGE pq_dw AS

    PROCEDURE addUniversidad (
        p_id universidad.id%TYPE
    );

    PROCEDURE updUniversidad (
        p_id universidad.id%TYPE,
        p_nombre universidad.nombre%TYPE
    );

    PROCEDURE addTitulacion (
        p_codigo titulacion.codigo%TYPE
    );

    PROCEDURE updTitulacion (
        p_codigo titulacion.codigo%TYPE,
        p_nombre titulacion.nombre%TYPE
    );
	
	PROCEDURE addProfesor (
		p_id profesor_responsable.id%TYPE
	);
	
	PROCEDURE updProfesor (
		p_id profesor_responsable.id%TYPE,
		p_nombreCompleto re_any_actual.profesor_max_stdnts%TYPE
	);

    PROCEDURE addEmpresa (
        p_id empresa.id%TYPE
    );

    PROCEDURE updEmpresa (
        p_id empresa.id%TYPE,
        p_nombre empresa.nombre%TYPE
    );
	
	PROCEDURE addOferta (
		p_empresa_id oferta.empresa_id%TYPE,
		p_plazas oferta.plazas%TYPE
	);
	
	PROCEDURE updOferta (
		p_empresa_id oferta.empresa_id%TYPE,
		p_var_plazas NUMBER
	);

	PROCEDURE delOferta (
		p_empresa_id oferta.empresa_id%TYPE,
		p_plazas oferta.plazas%TYPE
	);
	
	PROCEDURE addCandidatura;
	
	PROCEDURE delCandidatura;
	
	PROCEDURE addEntrevista (
		p_empresa_id entrevista.empresa_id%TYPE
	);

	PROCEDURE delEntrevista (
		p_empresa_id entrevista.empresa_id%TYPE
	);
	
	PROCEDURE addColaboracion (
		p_universidad_id colaboracion.universidad_id%TYPE,
		p_profesor_id cursa.profesor_id%TYPE,
		p_sueldo colaboracion.sueldo%TYPE,
		p_duracion NUMBER,
		p_informes_esperados colaboracion.informes_esperados%TYPE
	);

	PROCEDURE updColaboracion (
		p_var_sueldo NUMBER,
		p_var_duracion NUMBER,
		p_var_informes NUMBER
	);
	
	PROCEDURE delColaboracion (
		p_universidad_id colaboracion.universidad_id%TYPE,
		p_profesor_id cursa.profesor_id%TYPE,
		p_sueldo colaboracion.sueldo%TYPE,
		p_duracion NUMBER,
		p_informes_esperados colaboracion.informes_esperados%TYPE
	);
	
	PROCEDURE addInspeccion (
		p_empresa_id inspeccion.empresa_id%TYPE
	);
	
	PROCEDURE delInspeccion (
		p_empresa_id inspeccion.empresa_id%TYPE
	);
	
	PROCEDURE addInforme;
	
	PROCEDURE delInforme;
	
	PROCEDURE addInformeFinal (
        p_universidad_id informe_final.universidad_id%TYPE,
		p_titulacion_id cursa.titulacion_id%TYPE,
        p_es_apto informe_final.es_apto%TYPE
	);
	
	PROCEDURE updInformeFinal (
        p_universidad_id informe_final.universidad_id%TYPE,
		p_titulacion_id cursa.titulacion_id%TYPE,
        p_es_apto informe_final.es_apto%TYPE
	);
	
	PROCEDURE delInformeFinal (
        p_universidad_id informe_final.universidad_id%TYPE,
		p_titulacion_id cursa.titulacion_id%TYPE,
        p_es_apto informe_final.es_apto%TYPE
	);
	
	PROCEDURE resetCurso (
		p_anyo_univ re_any_univ.anyo_univ%TYPE
	);

END pq_dw;
/

CREATE OR REPLACE PACKAGE BODY pq_dw AS

    FUNCTION nom_empresas (
        p_ids re_general.empresas_mas_plazas_id%TYPE
    ) RETURN re_general.empresas_mas_plazas%TYPE IS
        v_nombres re_general.empresas_mas_plazas%TYPE;
    BEGIN
        SELECT LISTAGG(nombre, ', ')
        INTO v_nombres
        FROM empresa
        WHERE ',' || p_ids || ',' LIKE '%,' || id  || ',%'
        ORDER BY id;
        
        RETURN v_nombres;
    END nom_empresas;

    PROCEDURE addUniversidad (
        p_id universidad.id%TYPE
    ) IS
        operacion logs_dw.operacion%TYPE := 'pq_dw.addUniversidad';
        parametros logs_dw.parametros%TYPE := 'p_id = ' || p_id;
    BEGIN
        INSERT INTO re_universidades
        VALUES (p_id, 0, 0, 0);

        UPDATE re_any_actual
        SET total_universidades = total_universidades + 1;
    EXCEPTION
        WHEN OTHERS THEN pq_log.guardaLogDw (operacion, parametros, SUBSTR(SQLERRM, 1, 1000));
    END addUniversidad;

    PROCEDURE updUniversidad (
        p_id universidad.id%TYPE,
        p_nombre universidad.nombre%TYPE
    ) IS
        operacion logs_dw.operacion%TYPE := 'pq_dw.updUniversidad';
        parametros logs_dw.parametros%TYPE := 'p_id = ' || p_id || ', p_nombre = ''' || p_nombre || '''';
    BEGIN
        UPDATE re_general
        SET univ_max_stdnts_sup = p_nombre
        WHERE univ_max_stdnts_sup_id = p_id;
    EXCEPTION
        WHEN OTHERS THEN pq_log.guardaLogDw (operacion, parametros, SUBSTR(SQLERRM, 1, 1000));
    END updUniversidad;

    PROCEDURE addTitulacion (
        p_codigo titulacion.codigo%TYPE
    ) IS
        operacion logs_dw.operacion%TYPE := 'pq_dw.addTitulacion';
        parametros logs_dw.parametros%TYPE := 'p_codigo = ''' || p_codigo || '''';
    BEGIN
        INSERT INTO re_titulaciones
        VALUES (p_codigo, 0);
    EXCEPTION
        WHEN OTHERS THEN pq_log.guardaLogDw (operacion, parametros, SUBSTR(SQLERRM, 1, 1000));
    END addTitulacion;

    PROCEDURE updTitulacion (
        p_codigo titulacion.codigo%TYPE,
        p_nombre titulacion.nombre%TYPE
    ) IS
        operacion logs_dw.operacion%TYPE := 'pq_dw.updTitulacion';
        parametros logs_dw.parametros%TYPE := 'p_codigo = ''' || p_codigo || ''', p_nombre = ''' || p_nombre || '''';
    BEGIN
        UPDATE re_general
        SET titulacion = p_nombre
        WHERE titulacion_id = p_codigo;
    EXCEPTION
        WHEN OTHERS THEN pq_log.guardaLogDw (operacion, parametros, SUBSTR(SQLERRM, 1, 1000));
    END updTitulacion;

    PROCEDURE addProfesor (
        p_id profesor_responsable.id%TYPE
    ) IS
        operacion logs_dw.operacion%TYPE := 'pq_dw.addProfesor';
        parametros logs_dw.parametros%TYPE := 'p_id = ' || p_id;
    BEGIN
        INSERT INTO re_profesores
        VALUES (p_id, 0);
    EXCEPTION
        WHEN OTHERS THEN pq_log.guardaLogDw (operacion, parametros, SUBSTR(SQLERRM, 1, 1000));
    END addProfesor;
    
    PROCEDURE updProfesor (
        p_id profesor_responsable.id%TYPE,
        p_nombreCompleto re_any_actual.profesor_max_stdnts%TYPE
    ) IS
        operacion logs_dw.operacion%TYPE := 'pq_dw.updProfesor';
        parametros logs_dw.parametros%TYPE := 'p_id = ' || p_id || ', p_nombreCompleto = ''' || p_nombreCompleto || '''';
    BEGIN
        UPDATE re_any_actual
        SET profesor_max_stdnts = p_nombreCompleto
        WHERE profesor_max_stdnts_id = p_id;
    EXCEPTION
        WHEN OTHERS THEN pq_log.guardaLogDw (operacion, parametros, SUBSTR(SQLERRM, 1, 1000));
    END updProfesor;

    PROCEDURE addEmpresa (
        p_id empresa.id%TYPE
    ) IS
        operacion logs_dw.operacion%TYPE := 'pq_dw.addEmpresa';
        parametros logs_dw.parametros%TYPE := 'p_id = ' || p_id;
    BEGIN
        INSERT INTO re_empresas
        VALUES (p_id, 0, 0, 0, 0, 0);
    EXCEPTION
        WHEN OTHERS THEN pq_log.guardaLogDw (operacion, parametros, SUBSTR(SQLERRM, 1, 1000));
    END addEmpresa;
    
    PROCEDURE updEmpresa (
        p_id empresa.id%TYPE,
        p_nombre empresa.nombre%TYPE
    ) IS
        operacion logs_dw.operacion%TYPE := 'pq_dw.updEmpresa';
        parametros logs_dw.parametros%TYPE := 'p_id = ' || p_id || ', p_nombre = ''' || p_nombre || '''';
		--
        v_anyo_univ re_any_univ.anyo_univ%TYPE;
        v_empresas_mas_plazas_id re_general.empresas_mas_plazas_id%TYPE;
        v_empresas_mas_plazas re_general.empresas_mas_plazas%TYPE;
    BEGIN
        v_anyo_univ := anyoActual();
        UPDATE re_any_univ
        SET empresa_max_intrvws = p_nombre
        WHERE anyo_univ = v_anyo_univ
         AND empresa_max_intrvws_id = p_id;
        
        SELECT empresas_mas_plazas_id
        INTO v_empresas_mas_plazas_id
        FROM re_general;
        IF ',' || v_empresas_mas_plazas_id || ',' LIKE '%,' || p_id || ',%' THEN
            v_empresas_mas_plazas := nom_empresas(v_empresas_mas_plazas_id);
            UPDATE re_general
            SET empresas_mas_plazas = v_empresas_mas_plazas;
        END IF;
        
        UPDATE re_general
        SET empresa_mas_inspecciones = p_nombre
        WHERE empresa_mas_inspecciones_id = p_id;
    EXCEPTION
        WHEN OTHERS THEN pq_log.guardaLogDw (operacion, parametros, SUBSTR(SQLERRM, 1, 1000));
    END updEmpresa;
    
    PROCEDURE actualizaPlazasEmpresas (
        p_empresa_id oferta.empresa_id%TYPE,
        p_plazas NUMBER
    )
    IS
        v_empresas_mas_plazas_id re_general.empresas_mas_plazas_id%TYPE;
        v_empresas_mas_plazas re_general.empresas_mas_plazas%TYPE;
        v_prcnt_emp_no_plazas_any_act re_general.prcnt_emp_no_plazas_any_act%TYPE;
        v_num_empresas NUMBER;
        v_emp_sin_plazas NUMBER;
    BEGIN
        UPDATE re_empresas
        SET num_plazas = num_plazas + p_plazas,
         plazas_any_actual = plazas_any_actual + p_plazas
        WHERE id = p_empresa_id;

        SELECT LISTAGG(id, ','), LISTAGG(nombre, ', ')
        INTO v_empresas_mas_plazas_id, v_empresas_mas_plazas
        FROM 
            (SELECT e.id, e.nombre
            FROM re_empresas re
            JOIN empresa e ON re.id = e.id
            ORDER BY re.num_plazas DESC, e.id
            FETCH FIRST 5 ROWS ONLY);
        
        SELECT COUNT(1) total, SUM(CASE WHEN plazas_any_actual = 0 AND plazas_any_anterior > 0 THEN 1 ELSE 0 END) retiradas
        INTO v_num_empresas, v_emp_sin_plazas
        FROM re_empresas;
        
        v_prcnt_emp_no_plazas_any_act := v_emp_sin_plazas / NULLIF(v_num_empresas, 0) * 100;
        
        UPDATE re_general
        SET empresas_mas_plazas = v_empresas_mas_plazas,
         empresas_mas_plazas_id = v_empresas_mas_plazas_id,
         prcnt_emp_no_plazas_any_act = NVL(v_prcnt_emp_no_plazas_any_act, 0);         
    END actualizaPlazasEmpresas;

    PROCEDURE addOferta (
        p_empresa_id oferta.empresa_id%TYPE,
        p_plazas oferta.plazas%TYPE
    ) IS
        operacion logs_dw.operacion%TYPE := 'pq_dw.addOferta';
        parametros logs_dw.parametros%TYPE := 'p_empresa_id = ' || p_empresa_id || ', p_plazas = ' || p_plazas;
    BEGIN
        UPDATE re_any_actual
        SET num_ofertas = num_ofertas + 1,
         media_candidat_ofert = NVL(total_candidatos / NULLIF(num_ofertas + 1, 0), 0);
        
        actualizaPlazasEmpresas(p_empresa_id, p_plazas);
    EXCEPTION
        WHEN OTHERS THEN pq_log.guardaLogDw (operacion, parametros, SUBSTR(SQLERRM, 1, 1000));
    END addOferta;
    
    PROCEDURE updOferta (
        p_empresa_id oferta.empresa_id%TYPE,
        p_var_plazas NUMBER
    ) IS
        operacion logs_dw.operacion%TYPE := 'pq_dw.updOferta';
        parametros logs_dw.parametros%TYPE := 'p_empresa_id = ' || p_empresa_id || ', p_var_plazas = ' || p_var_plazas;
    BEGIN
        actualizaPlazasEmpresas(p_empresa_id, p_var_plazas);
    EXCEPTION
        WHEN OTHERS THEN pq_log.guardaLogDw (operacion, parametros, SUBSTR(SQLERRM, 1, 1000));
    END updOferta;
    
    PROCEDURE delOferta (
        p_empresa_id oferta.empresa_id%TYPE,
        p_plazas oferta.plazas%TYPE
    ) IS
        operacion logs_dw.operacion%TYPE := 'pq_dw.delOferta';
        parametros logs_dw.parametros%TYPE := 'p_empresa_id = ' || p_empresa_id || ', p_plazas = ' || p_plazas;
    BEGIN
        UPDATE re_any_actual
        SET num_ofertas = num_ofertas - 1,
         media_candidat_ofert = NVL(total_candidatos / NULLIF(num_ofertas - 1, 0), 0);
        
        actualizaPlazasEmpresas(p_empresa_id, -p_plazas);
    EXCEPTION
        WHEN OTHERS THEN pq_log.guardaLogDw (operacion, parametros, SUBSTR(SQLERRM, 1, 1000));
    END delOferta;
    
    PROCEDURE addCandidatura
    IS
        operacion logs_dw.operacion%TYPE := 'pq_dw.addCandidatura';
        parametros logs_dw.parametros%TYPE := ' ';
    BEGIN
        UPDATE re_any_actual
        SET total_candidatos = total_candidatos + 1,
         media_candidat_ofert = NVL((total_candidatos + 1) / NULLIF(num_ofertas, 0), 0);
    EXCEPTION
        WHEN OTHERS THEN pq_log.guardaLogDw (operacion, parametros, SUBSTR(SQLERRM, 1, 1000));
    END addCandidatura;
    
    PROCEDURE delCandidatura
    IS
        operacion logs_dw.operacion%TYPE := 'pq_dw.delCandidatura';
        parametros logs_dw.parametros%TYPE := ' ';
    BEGIN
        UPDATE re_any_actual
        SET total_candidatos = total_candidatos - 1,
         media_candidat_ofert = nvl((total_candidatos - 1) / NULLIF(num_ofertas, 0), 0);
    EXCEPTION
        WHEN OTHERS THEN pq_log.guardaLogDw (operacion, parametros, SUBSTR(SQLERRM, 1, 1000));
    END delCandidatura;
    
    PROCEDURE actualizaEntrevistas (
        p_empresa_id entrevista.empresa_id%TYPE,
        p_entrevistas NUMBER
    ) IS
		v_anyo_univ re_any_univ.anyo_univ%TYPE;
    BEGIN
        UPDATE re_empresas
        SET num_intrvws = num_intrvws + p_entrevistas
        WHERE id = p_empresa_id;

		v_anyo_univ := anyoActual();
        UPDATE re_any_univ
        SET (max_num_intrvws, empresa_max_intrvws_id, empresa_max_intrvws) =
            (SELECT re.num_intrvws, e.id, e.nombre
            FROM re_empresas re
             JOIN empresa e ON re.id = e.id
            ORDER BY re.num_intrvws DESC, e.id
            FETCH FIRST 1 ROWS ONLY)
		WHERE anyo_univ = v_anyo_univ;
    END actualizaEntrevistas;
    
    PROCEDURE addEntrevista (
        p_empresa_id entrevista.empresa_id%TYPE
    ) IS
        operacion logs_dw.operacion%TYPE := 'pq_dw.addEntrevista';
        parametros logs_dw.parametros%TYPE := 'p_empresa_id = ' || p_empresa_id;
    BEGIN
        actualizaEntrevistas(p_empresa_id, 1);
    EXCEPTION
        WHEN OTHERS THEN pq_log.guardaLogDw (operacion, parametros, SUBSTR(SQLERRM, 1, 1000));
    END addEntrevista;
    
    PROCEDURE delEntrevista (
        p_empresa_id entrevista.empresa_id%TYPE
    ) IS
        operacion logs_dw.operacion%TYPE := 'pq_dw.delEntrevista';
        parametros logs_dw.parametros%TYPE := 'p_empresa_id = ' || p_empresa_id;
    BEGIN
        actualizaEntrevistas(p_empresa_id, -1);
    EXCEPTION
        WHEN OTHERS THEN pq_log.guardaLogDw (operacion, parametros, SUBSTR(SQLERRM, 1, 1000));
    END delEntrevista;
    
    PROCEDURE actualizaColaboraciones (
        p_universidad_id colaboracion.universidad_id%TYPE,
        p_profesor_id cursa.profesor_id%TYPE,
        p_sueldo colaboracion.sueldo%TYPE,
        p_duracion NUMBER,
        p_informes_esperados colaboracion.informes_esperados%TYPE,
        p_unidad NUMBER
    ) IS
        v_anyo_univ re_any_univ.anyo_univ%TYPE;
        --
        v_num_univ_stdnts NUMBER;
        v_max_num_stdnts_prof re_any_actual.max_num_stdnts_prof%TYPE;
        v_profesor_max_stdnts_id re_any_actual.profesor_max_stdnts_id%TYPE;
        v_profesor_max_stdnts re_any_actual.profesor_max_stdnts%TYPE;
        --
        v_max_prcnt_stdnts_superan re_general.max_prcnt_stdnts_superan%TYPE;
        v_univ_max_stdnts_sup_id re_general.univ_max_stdnts_sup_id%TYPE;
        v_univ_max_stdnts_sup re_general.univ_max_stdnts_sup%TYPE;
    BEGIN
        UPDATE re_universidades
        SET stdnts_en_pract = stdnts_en_pract + p_unidad,
         prcnt_stdnts_sup = NVL(stdnts_superan / NULLIF(stdnts_en_pract + p_unidad, 0) * 100, 0)
        WHERE id = p_universidad_id;
        
        UPDATE re_profesores
        SET num_students = num_students + p_unidad
        WHERE id = p_profesor_id;
        
        v_anyo_univ := anyoActual();
        UPDATE re_any_univ
        SET total_students = total_students + p_unidad,
         prcnt_no_superan = NVL(no_superan / NULLIF(total_students + p_unidad, 0) * 100, 0)
        WHERE anyo_univ = v_anyo_univ;
        
        SELECT COUNT(DISTINCT c.universidad_id)
        INTO v_num_univ_stdnts
        FROM colaboracion c
         JOIN oferta o ON c.oferta_id = o.id
        WHERE o.anyo_acad = v_anyo_univ;
        
        SELECT re.num_students, p.id, TRIM(p.nombre || ' ' || p.apellido1 || ' ' || p.apellido2)
        INTO v_max_num_stdnts_prof, v_profesor_max_stdnts_id, v_profesor_max_stdnts
        FROM re_profesores re
         JOIN profesor_responsable p ON re.id = p.id
        ORDER BY re.num_students DESC, p.id
        FETCH FIRST 1 ROWS ONLY;
                
        UPDATE re_any_actual
        SET suma_sueldos = suma_sueldos + p_sueldo,
         num_muestras = num_muestras + p_unidad,
         sueldo_medio = NVL((suma_sueldos + p_sueldo) / NULLIF(num_muestras + p_unidad, 0), 0),
         num_estudiantes_actual = num_estudiantes_actual + p_unidad,
         num_univ_no_stdnts = total_universidades - v_num_univ_stdnts,
         prcnt_no_stdnts = NVL((total_universidades - v_num_univ_stdnts) / NULLIF(total_universidades, 0) * 100, 0),
         max_num_stdnts_prof = v_max_num_stdnts_prof,
         profesor_max_stdnts_id = v_profesor_max_stdnts_id,
         profesor_max_stdnts = v_profesor_max_stdnts;
        
        SELECT re.prcnt_stdnts_sup, u.id, u.nombre
        INTO v_max_prcnt_stdnts_superan, v_univ_max_stdnts_sup_id, v_univ_max_stdnts_sup
        FROM re_universidades re
         JOIN universidad u ON re.id = u.id
        ORDER BY re.prcnt_stdnts_sup DESC, u.id
        FETCH FIRST 1 ROWS ONLY;
        
        UPDATE re_general
        SET total_duracion_contratos = total_duracion_contratos + p_duracion,
         num_contratos = num_contratos + p_unidad,
         duracion_media_contrato = NVL((total_duracion_contratos + p_duracion) / NULLIF(num_contratos + p_unidad, 0), 0),
         num_informes_pendents = num_informes_pendents + p_informes_esperados,
         max_prcnt_stdnts_superan = v_max_prcnt_stdnts_superan,
         univ_max_stdnts_sup_id = v_univ_max_stdnts_sup_id,
         univ_max_stdnts_sup = v_univ_max_stdnts_sup;
    END actualizaColaboraciones;
    
    PROCEDURE addColaboracion (
        p_universidad_id colaboracion.universidad_id%TYPE,
        p_profesor_id cursa.profesor_id%TYPE,
        p_sueldo colaboracion.sueldo%TYPE,
        p_duracion NUMBER,
        p_informes_esperados colaboracion.informes_esperados%TYPE
    ) IS
        operacion logs_dw.operacion%TYPE := 'pq_dw.addColaboracion';
        parametros logs_dw.parametros%TYPE := 'p_universidad_id = ' || p_universidad_id || 'p_profesor_id = ' || p_profesor_id || 'p_sueldo = ' || p_sueldo || 'p_duracion = ' || p_duracion || 'p_informes_esperados = ' || p_informes_esperados;
    BEGIN
        actualizaColaboraciones(p_universidad_id, p_profesor_id, p_sueldo, p_duracion, p_informes_esperados, 1);
    EXCEPTION
        WHEN OTHERS THEN pq_log.guardaLogDw (operacion, parametros, SUBSTR(SQLERRM, 1, 1000));
    END addColaboracion;

    PROCEDURE updColaboracion (
        p_var_sueldo NUMBER,
        p_var_duracion NUMBER,
        p_var_informes NUMBER
    ) IS
        operacion logs_dw.operacion%TYPE := 'pq_dw.updColaboracion';
        parametros logs_dw.parametros%TYPE := 'p_var_sueldo = ' || p_var_sueldo || 'p_var_duracion = ' || p_var_duracion || 'p_var_informes = ' || p_var_informes;
    BEGIN
        UPDATE re_any_actual
        SET suma_sueldos = suma_sueldos + p_var_sueldo,
         sueldo_medio = NVL((suma_sueldos + p_var_sueldo) / NULLIF(num_muestras, 0), 0);
        
        UPDATE re_general
        SET total_duracion_contratos = total_duracion_contratos + p_var_duracion,
         duracion_media_contrato = NVL((total_duracion_contratos + p_var_duracion) / NULLIF(num_contratos, 0), 0),
         num_informes_pendents = num_informes_pendents + p_var_informes;
    EXCEPTION
        WHEN OTHERS THEN pq_log.guardaLogDw (operacion, parametros, SUBSTR(SQLERRM, 1, 1000));
    END updColaboracion;
    
    PROCEDURE delColaboracion (
        p_universidad_id colaboracion.universidad_id%TYPE,
        p_profesor_id cursa.profesor_id%TYPE,
        p_sueldo colaboracion.sueldo%TYPE,
        p_duracion NUMBER,
        p_informes_esperados colaboracion.informes_esperados%TYPE
    ) IS
        operacion logs_dw.operacion%TYPE := 'pq_dw.delColaboracion';
        parametros logs_dw.parametros%TYPE := 'p_universidad_id = ' || p_universidad_id || 'p_profesor_id = ' || p_profesor_id || 'p_sueldo = ' || p_sueldo || 'p_duracion = ' || p_duracion || 'p_informes_esperados = ' || p_informes_esperados;
    BEGIN
        actualizaColaboraciones(p_universidad_id, p_profesor_id, -p_sueldo, -p_duracion, -p_informes_esperados, -1);
    EXCEPTION
        WHEN OTHERS THEN pq_log.guardaLogDw (operacion, parametros, SUBSTR(SQLERRM, 1, 1000));
    END delColaboracion;
    
    PROCEDURE actualizaInspecciones (
        p_empresa_id inspeccion.empresa_id%TYPE,
        p_unidad NUMBER
    ) IS
        v_max_inspecciones  re_general.max_inspecciones%TYPE;
        v_empresa_mas_inspecciones_id  re_general.empresa_mas_inspecciones_id%TYPE;
        v_empresa_mas_inspecciones re_general.empresa_mas_inspecciones%TYPE;
    BEGIN
        UPDATE re_empresas
        SET inspecciones = inspecciones + p_unidad
        WHERE id = p_empresa_id;
        
        SELECT re.inspecciones, e.id, e.nombre
        INTO v_max_inspecciones, v_empresa_mas_inspecciones_id, v_empresa_mas_inspecciones
        FROM re_empresas re
         JOIN empresa e ON re.id = e.id
        ORDER BY re.inspecciones DESC, e.id
        FETCH FIRST 1 ROWS ONLY;
        
        UPDATE re_general
        SET max_inspecciones = v_max_inspecciones,
         empresa_mas_inspecciones_id = v_empresa_mas_inspecciones_id,
         empresa_mas_inspecciones = v_empresa_mas_inspecciones;
    END actualizaInspecciones;
    
    PROCEDURE addInspeccion (
        p_empresa_id inspeccion.empresa_id%TYPE
    ) IS
        operacion logs_dw.operacion%TYPE := 'pq_dw.addInspeccion';
        parametros logs_dw.parametros%TYPE := 'p_empresa_id = ' || p_empresa_id;
    BEGIN
        actualizaInspecciones(p_empresa_id, 1);
    EXCEPTION
        WHEN OTHERS THEN pq_log.guardaLogDw (operacion, parametros, SUBSTR(SQLERRM, 1, 1000));
    END addInspeccion;
    
    PROCEDURE delInspeccion (
        p_empresa_id inspeccion.empresa_id%TYPE
    ) IS
        operacion logs_dw.operacion%TYPE := 'pq_dw.delInspeccion';
        parametros logs_dw.parametros%TYPE := 'p_empresa_id = ' || p_empresa_id;
    BEGIN
        actualizaInspecciones(p_empresa_id, -1);
    EXCEPTION
        WHEN OTHERS THEN pq_log.guardaLogDw (operacion, parametros, SUBSTR(SQLERRM, 1, 1000));
    END delInspeccion;
    
    PROCEDURE addInforme IS
        operacion logs_dw.operacion%TYPE := 'pq_dw.addInforme';
        parametros logs_dw.parametros%TYPE := ' ';
    BEGIN
        UPDATE re_general
        SET num_informes_pendents = num_informes_pendents - 1;
    EXCEPTION
        WHEN OTHERS THEN pq_log.guardaLogDw (operacion, parametros, SUBSTR(SQLERRM, 1, 1000));
    END addInforme;
    
    PROCEDURE delInforme IS
        operacion logs_dw.operacion%TYPE := 'pq_dw.delInforme';
        parametros logs_dw.parametros%TYPE := ' ';
    BEGIN
        UPDATE re_general
        SET num_informes_pendents = num_informes_pendents + 1;
    EXCEPTION
        WHEN OTHERS THEN pq_log.guardaLogDw (operacion, parametros, SUBSTR(SQLERRM, 1, 1000));
    END delInforme;
    
    PROCEDURE actualizaInformesFinales (
        p_universidad_id informe_final.universidad_id%TYPE,
        p_titulacion_id cursa.titulacion_id%TYPE,
        p_unidad NUMBER
    ) IS
        v_max_prcnt_stdnts_superan re_general.max_prcnt_stdnts_superan%TYPE;
        v_univ_max_stdnts_sup_id re_general.univ_max_stdnts_sup_id%TYPE;
        v_univ_max_stdnts_sup re_general.univ_max_stdnts_sup%TYPE;
        v_max_stdnts_aptos re_general.max_stdnts_aptos%TYPE;
        v_titulacion_id re_general.titulacion_id%TYPE;
        v_titulacion re_general.titulacion%TYPE;
    BEGIN
        UPDATE re_universidades
        SET stdnts_superan = stdnts_superan + p_unidad,
         prcnt_stdnts_sup = NVL((stdnts_superan + p_unidad) / NULLIF(stdnts_en_pract, 0) *100, 0)
        WHERE id = p_universidad_id;
        
        UPDATE re_titulaciones
        SET estudiantes_aptos = estudiantes_aptos + p_unidad
        WHERE codigo = p_titulacion_id;
        
        SELECT re.prcnt_stdnts_sup, u.id, u.nombre
        INTO v_max_prcnt_stdnts_superan, v_univ_max_stdnts_sup_id, v_univ_max_stdnts_sup
        FROM re_universidades re
         JOIN universidad u ON re.id = u.id
        ORDER BY re.prcnt_stdnts_sup DESC, u.id
        FETCH FIRST 1 ROWS ONLY;
        
        SELECT re.estudiantes_aptos, t.codigo, t.nombre
        INTO v_max_stdnts_aptos, v_titulacion_id, v_titulacion
        FROM re_titulaciones re
         JOIN titulacion t ON re.codigo = t.codigo
        ORDER BY re.estudiantes_aptos DESC, t.codigo
        FETCH FIRST 1 ROWS ONLY;
        
        UPDATE re_general
        SET max_prcnt_stdnts_superan = v_max_prcnt_stdnts_superan,
         univ_max_stdnts_sup_id = v_univ_max_stdnts_sup_id,
         univ_max_stdnts_sup = v_univ_max_stdnts_sup,
         max_stdnts_aptos = v_max_stdnts_aptos,
         titulacion_id = v_titulacion_id,
         titulacion = v_titulacion;
    END actualizaInformesFinales;
    
    PROCEDURE addInformeFinal (
        p_universidad_id informe_final.universidad_id%TYPE,
        p_titulacion_id cursa.titulacion_id%TYPE,
        p_es_apto informe_final.es_apto%TYPE
    ) IS
        operacion logs_dw.operacion%TYPE := 'pq_dw.addInformeFinal';
        parametros logs_dw.parametros%TYPE := 'p_universidad_id = ' || p_universidad_id || ', p_titulacion_id = ''' || p_titulacion_id || ''', p_es_apto = ''' || p_es_apto || '''';
		--
		v_anyo_univ re_any_univ.anyo_univ%TYPE;
    BEGIN
        UPDATE re_any_actual
        SET num_estudiantes_actual = num_estudiantes_actual - 1;
        
        IF p_es_apto = '0' THEN
			v_anyo_univ := anyoActual();
            UPDATE re_any_univ
            SET no_superan = no_superan + 1,
             prcnt_no_superan = NVL((no_superan + 1) / NULLIF(total_students, 0) * 100, 0)
			WHERE anyo_univ = v_anyo_univ;
        ELSE
            actualizaInformesFinales(p_universidad_id, p_titulacion_id, 1);
        END IF;
    EXCEPTION
        WHEN OTHERS THEN pq_log.guardaLogDw (operacion, parametros, SUBSTR(SQLERRM, 1, 1000));
    END addInformeFinal;

    PROCEDURE updInformeFinal (
        p_universidad_id informe_final.universidad_id%TYPE,
        p_titulacion_id cursa.titulacion_id%TYPE,
        p_es_apto informe_final.es_apto%TYPE
    ) IS
        operacion logs_dw.operacion%TYPE := 'pq_dw.updInformeFinal';
        parametros logs_dw.parametros%TYPE := 'p_universidad_id = ' || p_universidad_id || ', p_titulacion_id = ''' || p_titulacion_id || ''', p_es_apto = ''' || p_es_apto || '''';
		--
        v_no_es_apto informe_final.es_apto%TYPE;
    BEGIN
        v_no_es_apto := TO_CHAR(ABS(TO_NUMBER(p_es_apto) - 1));
        
        delInformeFinal(p_universidad_id, p_titulacion_id, v_no_es_apto);
        addInformeFinal(p_universidad_id, p_titulacion_id, p_es_apto);
    EXCEPTION
        WHEN OTHERS THEN pq_log.guardaLogDw (operacion, parametros, SUBSTR(SQLERRM, 1, 1000));
    END updInformeFinal;
    
    PROCEDURE delInformeFinal (
        p_universidad_id informe_final.universidad_id%TYPE,
        p_titulacion_id cursa.titulacion_id%TYPE,
        p_es_apto informe_final.es_apto%TYPE
    ) IS
        operacion logs_dw.operacion%TYPE := 'pq_dw.delInformeFinal';
        parametros logs_dw.parametros%TYPE := 'p_universidad_id = ' || p_universidad_id || ', p_titulacion_id = ''' || p_titulacion_id || ''', p_es_apto = ''' || p_es_apto || '''';
		--
		v_anyo_univ re_any_univ.anyo_univ%TYPE;
    BEGIN
        UPDATE re_any_actual
        SET num_estudiantes_actual = num_estudiantes_actual + 1;
        
        IF p_es_apto = '0' THEN
			v_anyo_univ := anyoActual();
            UPDATE re_any_univ
            SET no_superan = no_superan - 1,
             prcnt_no_superan = NVL((no_superan - 1) / NULLIF(total_students, 0) * 100, 0)
			WHERE anyo_univ = v_anyo_univ;
        ELSE
            actualizaInformesFinales(p_universidad_id, p_titulacion_id, -1);
        END IF;
    EXCEPTION
        WHEN OTHERS THEN pq_log.guardaLogDw (operacion, parametros, SUBSTR(SQLERRM, 1, 1000));
    END delInformeFinal;
    
    PROCEDURE resetCurso (
        p_anyo_univ re_any_univ.anyo_univ%TYPE
    ) IS
        operacion logs_dw.operacion%TYPE := 'pq_dw.resetCurso';
        parametros logs_dw.parametros%TYPE := 'p_anyo_univ = ''' || p_anyo_univ || '''';
		--
        v_num NUMBER;
    BEGIN
        INSERT INTO re_any_univ
        VALUES (p_anyo_univ, 0, 0, 0, 0, NULL, NULL);
        
        SELECT COUNT(1)
        INTO v_num
        FROM universidad;
        
        UPDATE re_any_actual
        SET suma_sueldos = 0,
         num_muestras = 0,
         sueldo_medio = 0,
         num_estudiantes_actual = 0,
         total_universidades = v_num,
         num_univ_no_stdnts = v_num,
         prcnt_no_stdnts = 100,
         total_candidatos = 0,
         num_ofertas = 0,
         media_candidat_ofert = 0,
         max_num_stdnts_prof = 0,
         profesor_max_stdnts_id = NULL,
         profesor_max_stdnts = NULL;
		
		UPDATE re_general
		SET prcnt_emp_no_plazas_any_act = 100;
        
        UPDATE re_empresas
        SET plazas_any_anterior = plazas_any_actual,
         num_intrvws = 0,
         plazas_any_actual = 0;
		 
		 UPDATE re_profesores
		 SET num_students = 0;
    EXCEPTION
        WHEN OTHERS THEN pq_log.guardaLogDw (operacion, parametros, SUBSTR(SQLERRM, 1, 1000));
    END resetCurso;

END pq_dw;
