CREATE OR REPLACE PACKAGE pq_abm AS

    e_datos_obligatorios EXCEPTION;
    e_clave_primaria EXCEPTION;
    e_clave_foranea EXCEPTION;
    e_cascade EXCEPTION;
	e_duracion_baja EXCEPTION;
	
	c_datos_obligatorios CONSTANT logs.resultado%TYPE := 'ERROR FALTAN DATOS OBLIGATORIOS';
	c_clave_primaria CONSTANT logs.resultado%TYPE := 'ERROR NO EXISTE LA CLAVE PRIMARIA';
	c_clave_foranea CONSTANT logs.resultado%TYPE := 'ERROR NO EXISTE LA CLAVE FORÁNEA';
	c_cascade CONSTANT logs.resultado%TYPE := 'ERROR EXISTEN REGISTROS VINCULADOS';
	c_duracion_baja CONSTANT logs.resultado%TYPE := 'ERROR LA COLABORACIÓN NO PUEDE SER INFERIOR A UN MES';
	
    FUNCTION altaUniversidad (
        p_nombre universidad.nombre%TYPE,
        p_direccion universidad.direccion%TYPE,
        p_web universidad.web%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION modUniversidad (
        p_id universidad.id%TYPE,
        p_nombre universidad.nombre%TYPE,
        p_direccion universidad.direccion%TYPE,
        p_web universidad.web%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION altaTitulacion (
        p_codigo titulacion.codigo%TYPE,
        p_nombre titulacion.nombre%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION modTitulacion (
        p_codigo titulacion.codigo%TYPE,
        p_nombre titulacion.nombre%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION altaProfesor (
        p_universidad_id profesor_responsable.universidad_id%TYPE,
        p_nombre profesor_responsable.nombre%TYPE,
        p_apellido1 profesor_responsable.apellido1%TYPE,
        p_apellido2 profesor_responsable.apellido2%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION modProfesor (
        p_universidad_id profesor_responsable.universidad_id%TYPE,
		p_id profesor_responsable.id%TYPE,
        p_nombre profesor_responsable.nombre%TYPE,
        p_apellido1 profesor_responsable.apellido1%TYPE,
        p_apellido2 profesor_responsable.apellido2%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION altaEstudiante (
        p_universidad_id estudiante.universidad_id%TYPE,
        p_nombre estudiante.nombre%TYPE,
        p_apellido1 estudiante.apellido1%TYPE,
        p_apellido2 estudiante.apellido2%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION modEstudiante (
        p_universidad_id estudiante.universidad_id%TYPE,
		p_id estudiante.id%TYPE,
        p_nombre estudiante.nombre%TYPE,
        p_apellido1 estudiante.apellido1%TYPE,
        p_apellido2 estudiante.apellido2%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION altaCursa (
        p_universidad_id cursa.universidad_id%TYPE,
        p_titulacion_id cursa.titulacion_id%TYPE,
        p_profesor_id cursa.profesor_id%TYPE,
        p_estudiante_id cursa.estudiante_id%TYPE,
        p_anyo_acad cursa.anyo_acad%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION bajaCursa (
        p_universidad_id cursa.universidad_id%TYPE,
        p_titulacion_id cursa.titulacion_id%TYPE,
        p_profesor_id cursa.profesor_id%TYPE,
        p_estudiante_id cursa.estudiante_id%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION altaEmpresa (
        p_cif empresa.cif%TYPE,
        p_nombre empresa.nombre%TYPE,
        p_direccion empresa.direccion%TYPE,
        p_telefono empresa.telefono%TYPE,
        p_responsable_rrhh empresa.responsable_rrhh%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION modEmpresa (
        p_id empresa.id%TYPE,
        p_cif empresa.cif%TYPE,
        p_nombre empresa.nombre%TYPE,
        p_direccion empresa.direccion%TYPE,
        p_telefono empresa.telefono%TYPE,
        p_responsable_rrhh empresa.responsable_rrhh%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION altaSolicitante (
        p_empresa_id solicitante.empresa_id%TYPE,
        p_nombre solicitante.nombre%TYPE,
        p_cargo solicitante.cargo%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION modSolicitante (
        p_empresa_id solicitante.empresa_id%TYPE,
        p_id solicitante.id%TYPE,
        p_nombre solicitante.nombre%TYPE,
        p_cargo solicitante.cargo%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION bajaSolicitante (
        p_empresa_id solicitante.empresa_id%TYPE,
        p_id solicitante.id%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION altaOferta (
        p_empresa_id oferta.empresa_id%TYPE,
        p_solicitante_id oferta.solicitante_id%TYPE,
        p_descripcion oferta.descripcion%TYPE,
        p_requisitos oferta.requisitos%TYPE,
        p_plazas oferta.plazas%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION modOferta (
        p_id oferta.id%TYPE,
        p_descripcion oferta.descripcion%TYPE,
        p_requisitos oferta.requisitos%TYPE,
        p_plazas oferta.plazas%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION bajaOferta (
        p_id oferta.id%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION altaCandidatura (
        p_universidad_id candidatura.universidad_id%TYPE,
        p_estudiante_id candidatura.estudiante_id%TYPE,
        p_oferta_id candidatura.oferta_id%TYPE,
        p_fecha candidatura.fecha%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION modCandidatura (
        p_universidad_id candidatura.universidad_id%TYPE,
        p_estudiante_id candidatura.estudiante_id%TYPE,
        p_oferta_id candidatura.oferta_id%TYPE,
        p_fecha candidatura.fecha%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION bajaCandidatura (
        p_universidad_id candidatura.universidad_id%TYPE,
        p_estudiante_id candidatura.estudiante_id%TYPE,
        p_oferta_id candidatura.oferta_id%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION altaEntrevista (
        p_universidad_id entrevista.universidad_id%TYPE,
        p_estudiante_id entrevista.estudiante_id%TYPE,
        p_oferta_id entrevista.oferta_id%TYPE,
        p_empresa_id entrevista.empresa_id%TYPE,
        p_solicitante_id entrevista.solicitante_id%TYPE,
        p_fecha entrevista.fecha%TYPE,
        p_comentarios entrevista.comentarios%TYPE,
        p_es_apto entrevista.es_apto%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION modEntrevista (
        p_universidad_id entrevista.universidad_id%TYPE,
        p_estudiante_id entrevista.estudiante_id%TYPE,
        p_oferta_id entrevista.oferta_id%TYPE,
        p_empresa_id entrevista.empresa_id%TYPE,
        p_solicitante_id entrevista.solicitante_id%TYPE,
        p_fecha entrevista.fecha%TYPE,
        p_comentarios entrevista.comentarios%TYPE,
        p_es_apto entrevista.es_apto%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION bajaEntrevista (
        p_universidad_id entrevista.universidad_id%TYPE,
        p_estudiante_id entrevista.estudiante_id%TYPE,
        p_oferta_id entrevista.oferta_id%TYPE,
        p_empresa_id entrevista.empresa_id%TYPE,
        p_solicitante_id entrevista.solicitante_id%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION altaColaboracion (
        p_empresa_id colaboracion.empresa_id%TYPE,
        p_universidad_id colaboracion.universidad_id%TYPE,
        p_estudiante_id colaboracion.estudiante_id%TYPE,
        p_oferta_id colaboracion.oferta_id%TYPE,
        p_fecha_inicio colaboracion.fecha_inicio%TYPE,
        p_fecha_final colaboracion.fecha_final%TYPE,
        p_sueldo colaboracion.sueldo%TYPE,
        p_informes_esperados colaboracion.informes_esperados%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION modColaboracion (
        p_empresa_id colaboracion.empresa_id%TYPE,
        p_universidad_id colaboracion.universidad_id%TYPE,
        p_estudiante_id colaboracion.estudiante_id%TYPE,
        p_oferta_id colaboracion.oferta_id%TYPE,
        p_fecha_inicio colaboracion.fecha_inicio%TYPE,
        p_fecha_final colaboracion.fecha_final%TYPE,
        p_sueldo colaboracion.sueldo%TYPE,
        p_informes_esperados colaboracion.informes_esperados%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION bajaColaboracion (
        p_empresa_id colaboracion.empresa_id%TYPE,
        p_universidad_id colaboracion.universidad_id%TYPE,
        p_estudiante_id colaboracion.estudiante_id%TYPE,
        p_oferta_id colaboracion.oferta_id%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION altaInspeccion (
        p_empresa_id inspeccion.empresa_id%TYPE,
        p_fecha inspeccion.fecha%TYPE,
        p_muestreos inspeccion.muestreos%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION modInspeccion (
        p_id inspeccion.id%TYPE,
        p_fecha inspeccion.fecha%TYPE,
        p_muestreos inspeccion.muestreos%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION bajaInspeccion (
        p_id inspeccion.id%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION altaInforme (
        p_empresa_id informe.empresa_id%TYPE,
        p_universidad_id informe.universidad_id%TYPE,
        p_estudiante_id informe.estudiante_id%TYPE,
        p_oferta_id informe.oferta_id%TYPE,
        p_solicitante_id informe.solicitante_id%TYPE,
        p_fecha informe.fecha%TYPE,
        p_comentarios informe.comentarios%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION modInforme (
        p_empresa_id informe.empresa_id%TYPE,
        p_universidad_id informe.universidad_id%TYPE,
        p_estudiante_id informe.estudiante_id%TYPE,
        p_oferta_id informe.oferta_id%TYPE,
        p_solicitante_id informe.solicitante_id%TYPE,
        p_fecha informe.fecha%TYPE,
        p_comentarios informe.comentarios%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION bajaInforme (
        p_empresa_id informe.empresa_id%TYPE,
        p_universidad_id informe.universidad_id%TYPE,
        p_estudiante_id informe.estudiante_id%TYPE,
        p_oferta_id informe.oferta_id%TYPE,
        p_solicitante_id informe.solicitante_id%TYPE,
        p_fecha informe.fecha%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION altaInformeFinal (
        p_empresa_id informe_final.empresa_id%TYPE,
        p_universidad_id informe_final.universidad_id%TYPE,
        p_estudiante_id informe_final.estudiante_id%TYPE,
        p_oferta_id informe_final.oferta_id%TYPE,
        p_solicitante_id informe_final.solicitante_id%TYPE,
        p_fecha informe_final.fecha%TYPE,
        p_comentarios informe.comentarios%TYPE,
        p_valoracion informe_final.valoracion%TYPE,
        p_es_apto informe_final.es_apto%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION modInformeFinal (
        p_empresa_id informe_final.empresa_id%TYPE,
        p_universidad_id informe_final.universidad_id%TYPE,
        p_estudiante_id informe_final.estudiante_id%TYPE,
        p_oferta_id informe_final.oferta_id%TYPE,
        p_solicitante_id informe_final.solicitante_id%TYPE,
        p_fecha informe_final.fecha%TYPE,
        p_comentarios informe.comentarios%TYPE,
        p_valoracion informe_final.valoracion%TYPE,
        p_es_apto informe_final.es_apto%TYPE
    ) RETURN logs.resultado%TYPE;

    FUNCTION bajaInformeFinal (
        p_empresa_id informe_final.empresa_id%TYPE,
        p_universidad_id informe_final.universidad_id%TYPE,
        p_estudiante_id informe_final.estudiante_id%TYPE,
        p_oferta_id informe_final.oferta_id%TYPE,
        p_solicitante_id informe_final.solicitante_id%TYPE,
        p_fecha informe_final.fecha%TYPE
    ) RETURN logs.resultado%TYPE;

END pq_abm;
/

CREATE OR REPLACE PACKAGE BODY pq_abm AS

    FUNCTION logAndResult (
        operacion logs.operacion%TYPE,
        parametros logs.parametros%TYPE,
        resultado logs.resultado%TYPE
    )  RETURN logs.resultado%TYPE IS
    BEGIN
        pq_log.guardaLog(operacion, parametros, resultado);
        RETURN resultado;
    END logAndResult;

    FUNCTION altaUniversidad (
        p_nombre universidad.nombre%TYPE,
        p_direccion universidad.direccion%TYPE,
        p_web universidad.web%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.altaUniversidad';
        parametros logs.parametros%TYPE := 'p_nombre = ''' || p_nombre || ''', p_direccion = ''' || p_direccion || ''', p_web = ''' || p_web || '''';
		--
		v_id universidad.id%TYPE;
    BEGIN
        IF p_nombre IS NULL OR p_direccion IS NULL THEN
            RAISE e_datos_obligatorios;
        END IF;

        INSERT INTO universidad
        VALUES (NULL, p_nombre, p_direccion, p_web)
		RETURNING id INTO v_id;
		
		pq_dw.addUniversidad(v_id);

        RETURN logAndResult(operacion, parametros, 'OK');
    EXCEPTION
        WHEN e_datos_obligatorios THEN RETURN logAndResult(operacion, parametros, c_datos_obligatorios);
        WHEN DUP_VAL_ON_INDEX THEN RETURN logAndResult(operacion, parametros, 'ERROR NOMBRE DUPLICADO');
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
    END altaUniversidad;

    FUNCTION modUniversidad (
        p_id universidad.id%TYPE,
        p_nombre universidad.nombre%TYPE,
        p_direccion universidad.direccion%TYPE,
        p_web universidad.web%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.modUniversidad';
        parametros logs.parametros%TYPE := 'p_id = ' || p_id || ', p_nombre = ''' || p_nombre || ''', p_direccion = ''' || p_direccion || ''', p_web = ''' || p_web || '''';
        --
        CURSOR universidades IS
        SELECT *
        FROM universidad
        WHERE id = p_id
        FOR UPDATE;
        universidad universidades%ROWTYPE;
    BEGIN
        OPEN universidades;
        FETCH universidades INTO universidad;
        IF universidades%FOUND THEN
            IF p_nombre IS NULL OR p_direccion IS NULL THEN
                RAISE e_datos_obligatorios;
            END IF;
			
            UPDATE universidad
            SET nombre = p_nombre,
             direccion = p_direccion,
             web = p_web
            WHERE CURRENT OF universidades;
			
			IF universidad.nombre <> p_nombre THEN
                pq_dw.updUniversidad(p_id, p_nombre);
			END IF;
			
            RETURN logAndResult(operacion, parametros, 'OK');
        ELSE
            RAISE e_clave_primaria;
        END IF;
        CLOSE universidades;
    EXCEPTION
        WHEN e_clave_primaria THEN RETURN logAndResult(operacion, parametros, c_clave_primaria);
        WHEN e_datos_obligatorios THEN RETURN logAndResult(operacion, parametros, c_datos_obligatorios);
        WHEN DUP_VAL_ON_INDEX THEN RETURN logAndResult(operacion, parametros, 'ERROR NOMBRE DUPLICADO');
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
    END modUniversidad;

    FUNCTION altaTitulacion (
        p_codigo titulacion.codigo%TYPE,
        p_nombre titulacion.nombre%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.altaTitulacion';
        parametros logs.parametros%TYPE := 'p_codigo = ' || p_codigo || ', p_nombre = ''' || p_nombre || '''';
    BEGIN
        IF p_codigo IS NULL OR p_nombre IS NULL THEN
            RAISE e_datos_obligatorios;
        END IF;

        INSERT INTO titulacion
        VALUES (p_codigo, p_nombre);

        pq_dw.addTitulacion(p_codigo);

        RETURN logAndResult(operacion, parametros, 'OK');
    EXCEPTION
        WHEN e_datos_obligatorios THEN RETURN logAndResult(operacion, parametros, c_datos_obligatorios);
        WHEN DUP_VAL_ON_INDEX THEN RETURN logAndResult(operacion, parametros, 'ERROR CODIGO DUPLICADO');
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
    END altaTitulacion;

    FUNCTION modTitulacion (
        p_codigo titulacion.codigo%TYPE,
        p_nombre titulacion.nombre%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.modTitulacion';
        parametros logs.parametros%TYPE := 'p_codigo = ' || p_codigo || ', p_nombre = ''' || p_nombre || '''';
        --
        CURSOR titulaciones IS
        SELECT *
        FROM titulacion
        WHERE codigo = p_codigo
        FOR UPDATE;
        titulacion titulaciones%ROWTYPE;
    BEGIN
        OPEN titulaciones;
        FETCH titulaciones INTO titulacion;
        IF titulaciones%FOUND THEN
            IF p_nombre IS NULL THEN
                RAISE e_datos_obligatorios;
            END IF;

            UPDATE titulacion
            SET nombre = p_nombre
            WHERE CURRENT OF titulaciones;

            pq_dw.updTitulacion(p_codigo, p_nombre);

            RETURN logAndResult(operacion, parametros, 'OK');
        ELSE
            RAISE e_clave_primaria;
        END IF;
        CLOSE titulaciones;
    EXCEPTION
        WHEN e_clave_primaria THEN RETURN logAndResult(operacion, parametros, c_clave_primaria);
        WHEN e_datos_obligatorios THEN RETURN logAndResult(operacion, parametros, c_datos_obligatorios);
        WHEN DUP_VAL_ON_INDEX THEN RETURN logAndResult(operacion, parametros, 'ERROR CODIGO DUPLICADO');
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
    END modTitulacion;
	
    FUNCTION altaProfesor (
        p_universidad_id profesor_responsable.universidad_id%TYPE,
        p_nombre profesor_responsable.nombre%TYPE,
        p_apellido1 profesor_responsable.apellido1%TYPE,
        p_apellido2 profesor_responsable.apellido2%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.altaProfesor';
        parametros logs.parametros%TYPE := 'p_universidad_id = ' || p_universidad_id || ', p_nombre = ''' || p_nombre || ''', p_apellido1 = ''' || p_apellido1 || ''', p_apellido2 = ''' || p_apellido2 || '''';
		--
		v_num NUMBER;
		v_id profesor_responsable.id%TYPE;
	BEGIN
		SELECT COUNT(1)
		INTO v_num
		FROM universidad
		WHERE id = p_universidad_id;
		IF v_num = 0 THEN
			RAISE e_clave_foranea;
		END IF;
		
        IF p_universidad_id IS NULL OR p_nombre IS NULL OR p_apellido1 IS NULL THEN
            RAISE e_datos_obligatorios;
        END IF;

        INSERT INTO profesor_responsable
        VALUES (p_universidad_id, NULL, p_nombre, p_apellido1, p_apellido2)
		RETURNING id INTO v_id;

        pq_dw.addProfesor(v_id);

        RETURN logAndResult(operacion, parametros, 'OK');
    EXCEPTION
        WHEN e_clave_foranea THEN RETURN logAndResult(operacion, parametros, c_clave_foranea);
        WHEN e_datos_obligatorios THEN RETURN logAndResult(operacion, parametros, c_datos_obligatorios);
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
	END altaProfesor;

    FUNCTION modProfesor (
        p_universidad_id profesor_responsable.universidad_id%TYPE,
		p_id profesor_responsable.id%TYPE,
        p_nombre profesor_responsable.nombre%TYPE,
        p_apellido1 profesor_responsable.apellido1%TYPE,
        p_apellido2 profesor_responsable.apellido2%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.modProfesor';
        parametros logs.parametros%TYPE := 'p_universidad_id = ' || p_universidad_id || ', p_id = ' || p_id || ', p_nombre = ''' || p_nombre || ''', p_apellido1 = ''' || p_apellido1 || ''', p_apellido2 = ''' || p_apellido2 || '''';
        --
        CURSOR profesores IS
        SELECT *
        FROM profesor_responsable
        WHERE universidad_id = p_universidad_id
		 and id = p_id
        FOR UPDATE;
        profesor profesores%ROWTYPE;
    BEGIN
        OPEN profesores;
        FETCH profesores INTO profesor;
        IF profesores%FOUND THEN
            IF p_nombre IS NULL OR p_apellido1 IS NULL THEN
                RAISE e_datos_obligatorios;
            END IF;
			
            UPDATE profesor_responsable
            SET nombre = p_nombre,
             apellido1 = p_apellido1,
             apellido2 = p_apellido2
            WHERE CURRENT OF profesores;
			
            pq_dw.updProfesor(p_id, TRIM(p_nombre || ' ' || p_apellido1 || ' ' || p_apellido2));
			
            RETURN logAndResult(operacion, parametros, 'OK');
        ELSE
            RAISE e_clave_primaria;
        END IF;
        CLOSE profesores;
    EXCEPTION
        WHEN e_clave_primaria THEN RETURN logAndResult(operacion, parametros, c_clave_primaria);
        WHEN e_datos_obligatorios THEN RETURN logAndResult(operacion, parametros, c_datos_obligatorios);
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
    END modProfesor;

    FUNCTION altaEstudiante (
        p_universidad_id estudiante.universidad_id%TYPE,
        p_nombre estudiante.nombre%TYPE,
        p_apellido1 estudiante.apellido1%TYPE,
        p_apellido2 estudiante.apellido2%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.altaEstudiante';
        parametros logs.parametros%TYPE := 'p_universidad_id = ' || p_universidad_id || ', p_nombre = ''' || p_nombre || ''', p_apellido1 = ''' || p_apellido1 || ''', p_apellido2 = ''' || p_apellido2 || '''';
		--
		v_num NUMBER;
	BEGIN
		SELECT COUNT(1)
		INTO v_num
		FROM universidad
		WHERE id = p_universidad_id;
		IF v_num = 0 THEN
			RAISE e_clave_foranea;
		END IF;
		
        IF p_universidad_id IS NULL OR p_nombre IS NULL OR p_apellido1 IS NULL THEN
            RAISE e_datos_obligatorios;
        END IF;

        INSERT INTO estudiante
        VALUES (p_universidad_id, NULL, p_nombre, p_apellido1, p_apellido2);

        RETURN logAndResult(operacion, parametros, 'OK');
    EXCEPTION
        WHEN e_clave_foranea THEN RETURN logAndResult(operacion, parametros, c_clave_foranea);
        WHEN e_datos_obligatorios THEN RETURN logAndResult(operacion, parametros, c_datos_obligatorios);
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
	END altaEstudiante;

    FUNCTION modEstudiante (
        p_universidad_id estudiante.universidad_id%TYPE,
		p_id estudiante.id%TYPE,
        p_nombre estudiante.nombre%TYPE,
        p_apellido1 estudiante.apellido1%TYPE,
        p_apellido2 estudiante.apellido2%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.modEstudiante';
        parametros logs.parametros%TYPE := 'p_universidad_id = ' || p_universidad_id || ', p_id = ' || p_id || ', p_nombre = ''' || p_nombre || ''', p_apellido1 = ''' || p_apellido1 || ''', p_apellido2 = ''' || p_apellido2 || '''';
        --
        CURSOR estudiantes IS
        SELECT *
        FROM estudiante
        WHERE universidad_id = p_universidad_id
		 and id = p_id
        FOR UPDATE;
        estudiante estudiantes%ROWTYPE;
    BEGIN
        OPEN estudiantes;
        FETCH estudiantes INTO estudiante;
        IF estudiantes%FOUND THEN
            IF p_nombre IS NULL OR p_apellido1 IS NULL THEN
                RAISE e_datos_obligatorios;
            END IF;
			
            UPDATE estudiante
            SET nombre = p_nombre,
             apellido1 = p_apellido1,
             apellido2 = p_apellido2
            WHERE CURRENT OF estudiantes;
			
            RETURN logAndResult(operacion, parametros, 'OK');
        ELSE
            RAISE e_clave_primaria;
        END IF;
        CLOSE estudiantes;
    EXCEPTION
        WHEN e_clave_primaria THEN RETURN logAndResult(operacion, parametros, c_clave_primaria);
        WHEN e_datos_obligatorios THEN RETURN logAndResult(operacion, parametros, c_datos_obligatorios);
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
    END modEstudiante;
	
	FUNCTION altaCursa (
        p_universidad_id cursa.universidad_id%TYPE,
        p_titulacion_id cursa.titulacion_id%TYPE,
        p_profesor_id cursa.profesor_id%TYPE,
        p_estudiante_id cursa.estudiante_id%TYPE,
        p_anyo_acad cursa.anyo_acad%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.altaCursa';
        parametros logs.parametros%TYPE := 'p_universidad_id = ' || p_universidad_id || ', p_titulacion_id = ''' || p_titulacion_id || ''', p_profesor_id = ' || p_profesor_id || ', p_estudiante_id = ' || p_estudiante_id || ', p_anyo_acad = ''' || p_anyo_acad || '''';
		--
		v_num NUMBER;
	BEGIN
		SELECT COUNT(1)
		INTO v_num
		FROM profesor_responsable
		WHERE universidad_id = p_universidad_id
		 AND id = p_profesor_id;
		IF v_num = 0 THEN
			RAISE e_clave_foranea;
		END IF;
		
		SELECT COUNT(1)
		INTO v_num
		FROM estudiante
		WHERE universidad_id = p_universidad_id
		 AND id = p_estudiante_id;
		IF v_num = 0 THEN
			RAISE e_clave_foranea;
		END IF;
		
		SELECT COUNT(1)
		INTO v_num
		FROM titulacion
		WHERE codigo = p_titulacion_id;
		IF v_num = 0 THEN
			RAISE e_clave_foranea;
		END IF;
		
		IF p_anyo_acad IS NULL THEN
            RAISE e_datos_obligatorios;
        END IF;
		
		INSERT INTO cursa
		VALUES (p_universidad_id, p_titulacion_id, p_profesor_id, p_estudiante_id, p_anyo_acad);
		
        RETURN logAndResult(operacion, parametros, 'OK');
    EXCEPTION
        WHEN e_clave_foranea THEN RETURN logAndResult(operacion, parametros, c_clave_foranea);
        WHEN e_datos_obligatorios THEN RETURN logAndResult(operacion, parametros, c_datos_obligatorios);
        WHEN DUP_VAL_ON_INDEX THEN RETURN logAndResult(operacion, parametros, 'ERROR REGISTRO DUPLICADO');
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
	END altaCursa;

    FUNCTION bajaCursa (
        p_universidad_id cursa.universidad_id%TYPE,
        p_titulacion_id cursa.titulacion_id%TYPE,
        p_profesor_id cursa.profesor_id%TYPE,
        p_estudiante_id cursa.estudiante_id%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.bajaCursa';
        parametros logs.parametros%TYPE := 'p_universidad_id = ' || p_universidad_id || ', p_titulacion_id = ''' || p_titulacion_id || ''', p_profesor_id = ' || p_profesor_id || ', p_estudiante_id = ' || p_estudiante_id;
		--
		CURSOR cursas IS
		SELECT *
		FROM cursa
		WHERE universidad_id = p_universidad_id
		 and titulacion_id = p_titulacion_id
		 and profesor_id = p_profesor_id
		 and estudiante_id = p_estudiante_id
		FOR UPDATE;
		cursa cursas%ROWTYPE;
		--
		v_num NUMBER;
	BEGIN
		OPEN cursas;
		FETCH cursas INTO cursa;
		IF cursas%FOUND THEN
			SELECT COUNT(1)
			INTO v_num
			FROM candidatura c
			 JOIN oferta o ON c.oferta_id = o.id
			WHERE c.universidad_id = p_universidad_id
			 AND c.estudiante_id = p_estudiante_id
			 AND o.anyo_acad = cursa.anyo_acad;
			IF v_num > 0 THEN
				RAISE e_cascade;
			END IF;
			
			DELETE cursa
			WHERE CURRENT OF cursas;
			
            RETURN logAndResult(operacion, parametros, 'OK');
		ELSE
            RAISE e_clave_primaria;
        END IF;
		CLOSE cursas;
    EXCEPTION
        WHEN e_clave_primaria THEN RETURN logAndResult(operacion, parametros, c_clave_primaria);
        WHEN e_cascade THEN RETURN logAndResult(operacion, parametros, c_cascade);
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
	END bajaCursa;
	
	FUNCTION altaEmpresa (
        p_cif empresa.cif%TYPE,
        p_nombre empresa.nombre%TYPE,
        p_direccion empresa.direccion%TYPE,
        p_telefono empresa.telefono%TYPE,
        p_responsable_rrhh empresa.responsable_rrhh%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.altaEmpresa';
        parametros logs.parametros%TYPE := 'p_cif = ''' || p_cif || ''', p_nombre = ''' || p_nombre || ''', p_direccion = ''' || p_direccion || ''', p_telefono = ''' || p_telefono || ''', p_responsable_rrhh = ''' || p_responsable_rrhh || '''';
		--
		v_id empresa.id%TYPE;
    BEGIN
        IF p_cif IS NULL OR p_nombre IS NULL OR p_direccion IS NULL OR p_telefono IS NULL OR p_responsable_rrhh IS NULL THEN
            RAISE e_datos_obligatorios;
        END IF;
		
		INSERT INTO empresa
		VALUES (NULL, p_cif, p_nombre, p_direccion, p_telefono, p_responsable_rrhh)
		RETURNING id INTO v_id;
		
		pq_dw.addEmpresa(v_id);
		
        RETURN logAndResult(operacion, parametros, 'OK');
    EXCEPTION
        WHEN e_datos_obligatorios THEN RETURN logAndResult(operacion, parametros, c_datos_obligatorios);
        WHEN DUP_VAL_ON_INDEX THEN RETURN logAndResult(operacion, parametros, 'ERROR CIF DUPLICADO');
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
	END altaEmpresa;

    FUNCTION modEmpresa (
        p_id empresa.id%TYPE,
        p_cif empresa.cif%TYPE,
        p_nombre empresa.nombre%TYPE,
        p_direccion empresa.direccion%TYPE,
        p_telefono empresa.telefono%TYPE,
        p_responsable_rrhh empresa.responsable_rrhh%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.modEmpresa';
        parametros logs.parametros%TYPE := 'p_id = ' || p_id || ', p_cif = ''' || p_cif || ''', p_nombre = ''' || p_nombre || ''', p_direccion = ''' || p_direccion || ''', p_telefono = ''' || p_telefono || ''', p_responsable_rrhh = ''' || p_responsable_rrhh || '''';
		--
		CURSOR empresas IS
        SELECT *
        FROM empresa
        WHERE id = p_id
        FOR UPDATE;
        empresa empresas%ROWTYPE;
    BEGIN
        OPEN empresas;
        FETCH empresas INTO empresa;
        IF empresas%FOUND THEN
            IF p_cif IS NULL OR p_nombre IS NULL OR p_direccion IS NULL OR p_telefono IS NULL OR p_responsable_rrhh IS NULL THEN
                RAISE e_datos_obligatorios;
            END IF;
			
            UPDATE empresa
            SET cif = p_cif,
             nombre = p_nombre,
             direccion = p_direccion,
			 telefono = p_telefono,
			 responsable_rrhh = p_responsable_rrhh
            WHERE CURRENT OF empresas;
			
			IF empresa.nombre <> p_nombre THEN
                pq_dw.updEmpresa(p_id, p_nombre);
			END IF;
			
            RETURN logAndResult(operacion, parametros, 'OK');
        ELSE
            RAISE e_clave_primaria;
        END IF;
        CLOSE empresas;
    EXCEPTION
        WHEN e_clave_primaria THEN RETURN logAndResult(operacion, parametros, c_clave_primaria);
        WHEN e_datos_obligatorios THEN RETURN logAndResult(operacion, parametros, c_datos_obligatorios);
        WHEN DUP_VAL_ON_INDEX THEN RETURN logAndResult(operacion, parametros, 'ERROR NOMBRE DUPLICADO');
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
	END modEmpresa;

    FUNCTION altaSolicitante (
        p_empresa_id solicitante.empresa_id%TYPE,
        p_nombre solicitante.nombre%TYPE,
        p_cargo solicitante.cargo%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.altaSolicitante';
        parametros logs.parametros%TYPE := 'p_empresa_id = ' || p_empresa_id || ', p_nombre = ''' || p_nombre || ''', p_cargo = ''' || p_cargo || '''';
		--
		v_num NUMBER;
	BEGIN
		SELECT COUNT(1)
		INTO v_num
		FROM empresa
		WHERE id = p_empresa_id;
		IF v_num = 0 THEN
			RAISE e_clave_foranea;
		END IF;
		
		IF p_nombre IS NULL OR p_cargo IS NULL THEN
            RAISE e_datos_obligatorios;
        END IF;

        INSERT INTO solicitante
        VALUES (p_empresa_id, NULL, p_nombre, p_cargo);
		
        RETURN logAndResult(operacion, parametros, 'OK');
    EXCEPTION
        WHEN e_clave_foranea THEN RETURN logAndResult(operacion, parametros, c_clave_foranea);
        WHEN e_datos_obligatorios THEN RETURN logAndResult(operacion, parametros, c_datos_obligatorios);
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
	END altaSolicitante;

    FUNCTION modSolicitante (
        p_empresa_id solicitante.empresa_id%TYPE,
        p_id solicitante.id%TYPE,
        p_nombre solicitante.nombre%TYPE,
        p_cargo solicitante.cargo%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.modSolicitante';
        parametros logs.parametros%TYPE := 'p_empresa_id = ' || p_empresa_id || ', p_id = ' || p_id || ', p_nombre = ''' || p_nombre || ''', p_cargo = ''' || p_cargo || '''';
		--
		CURSOR solicitantes IS
		SELECT *
		FROM solicitante
		WHERE empresa_id = p_empresa_id
		 and id = p_id
		FOR UPDATE;
		solicitante solicitantes%ROWTYPE;
	BEGIN
        OPEN solicitantes;
        FETCH solicitantes INTO solicitante;
        IF solicitantes%FOUND THEN
            IF p_nombre IS NULL OR p_cargo IS NULL THEN
                RAISE e_datos_obligatorios;
            END IF;
			
            UPDATE solicitante
            SET nombre = p_nombre,
             cargo = p_cargo
            WHERE CURRENT OF solicitantes;
			
            RETURN logAndResult(operacion, parametros, 'OK');
        ELSE
            RAISE e_clave_primaria;
        END IF;
        CLOSE solicitantes;
    EXCEPTION
        WHEN e_clave_primaria THEN RETURN logAndResult(operacion, parametros, c_clave_primaria);
        WHEN e_datos_obligatorios THEN RETURN logAndResult(operacion, parametros, c_datos_obligatorios);
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
	END modSolicitante;

    FUNCTION bajaSolicitante (
        p_empresa_id solicitante.empresa_id%TYPE,
        p_id solicitante.id%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.bajaSolicitante';
        parametros logs.parametros%TYPE := 'p_empresa_id = ' || p_empresa_id || ', p_id = ' || p_id;
		--
		CURSOR solicitantes IS
		SELECT *
		FROM solicitante
		WHERE empresa_id = p_empresa_id
		 and id = p_id
		FOR UPDATE;
		solicitante solicitantes%ROWTYPE;
		--
		v_num NUMBER;
	BEGIN
        OPEN solicitantes;
        FETCH solicitantes INTO solicitante;
        IF solicitantes%FOUND THEN
            SELECT COUNT(1)
			INTO v_num
			FROM oferta
			WHERE empresa_id = p_empresa_id
			 AND solicitante_id = p_id;
			IF v_num > 0 THEN
				RAISE e_cascade;
			END IF;
			
            DELETE solicitante
            WHERE CURRENT OF solicitantes;
			
            RETURN logAndResult(operacion, parametros, 'OK');
        ELSE
            RAISE e_clave_primaria;
        END IF;
        CLOSE solicitantes;
    EXCEPTION
        WHEN e_clave_primaria THEN RETURN logAndResult(operacion, parametros, c_clave_primaria);
        WHEN e_cascade THEN RETURN logAndResult(operacion, parametros, c_cascade);
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
	END bajaSolicitante;

    FUNCTION altaOferta (
        p_empresa_id oferta.empresa_id%TYPE,
        p_solicitante_id oferta.solicitante_id%TYPE,
        p_descripcion oferta.descripcion%TYPE,
        p_requisitos oferta.requisitos%TYPE,
        p_plazas oferta.plazas%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.altaOferta';
        parametros logs.parametros%TYPE := 'p_empresa_id = ' || p_empresa_id || ', p_solicitante_id = ' || p_solicitante_id || ', p_descripcion = ''' || p_descripcion || ''', p_requisitos = ''' || p_requisitos || ''', p_plazas = ' || p_plazas;
		--
		v_num NUMBER;
		v_id oferta.id%TYPE;
	BEGIN
		SELECT COUNT(1)
		INTO v_num
		FROM solicitante
		WHERE empresa_id = p_empresa_id
		 AND id = p_solicitante_id;
		IF v_num = 0 THEN
			RAISE e_clave_foranea;
		END IF;
		
		IF p_descripcion IS NULL OR NVL(p_plazas, 0) < 1 THEN
			RAISE e_datos_obligatorios;
		END IF;
		
		INSERT INTO oferta
		VALUES (NULL, p_empresa_id, p_solicitante_id, p_descripcion, p_requisitos, anyoActual(), p_plazas)
		RETURNING id INTO v_id;
		
		INSERT INTO recibe
		SELECT id, v_id
		FROM universidad;
		
		pq_dw.addOferta(p_empresa_id, p_plazas);
		
		RETURN logAndResult(operacion, parametros, 'OK');
    EXCEPTION
        WHEN e_clave_foranea THEN RETURN logAndResult(operacion, parametros, c_clave_foranea);
        WHEN e_datos_obligatorios THEN RETURN logAndResult(operacion, parametros, c_datos_obligatorios);
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
	END altaOferta;

    FUNCTION modOferta (
        p_id oferta.id%TYPE,
        p_descripcion oferta.descripcion%TYPE,
        p_requisitos oferta.requisitos%TYPE,
        p_plazas oferta.plazas%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.modOferta';
        parametros logs.parametros%TYPE := 'p_id = ' || p_id || ', p_descripcion = ''' || p_descripcion || ''', p_requisitos = ''' || p_requisitos || ''', p_plazas = ' || p_plazas;
		--
		CURSOR ofertas IS
		SELECT *
		FROM oferta
		WHERE id = p_id
		FOR UPDATE;
		oferta ofertas%ROWTYPE;
		--
		v_num NUMBER;
	BEGIN
        OPEN ofertas;
        FETCH ofertas INTO oferta;
        IF ofertas%FOUND THEN
            IF p_descripcion IS NULL OR NVL(p_plazas, 0) < 1 THEN
                RAISE e_datos_obligatorios;
            END IF;
			
			IF p_plazas < oferta.plazas THEN
				SELECT COUNT(1)
				INTO v_num
				FROM colaboracion
				WHERE oferta_id = p_id;
				IF v_num > p_plazas THEN
					RAISE e_cascade;
				END IF;
			END IF;
			
            UPDATE oferta
            SET descripcion = p_descripcion,
             requisitos = p_requisitos,
			 plazas = p_plazas
            WHERE CURRENT OF ofertas;
			
			IF p_plazas <> oferta.plazas THEN
				pq_dw.updOferta(oferta.empresa_id, p_plazas - oferta.plazas);
			END IF;
			
            RETURN logAndResult(operacion, parametros, 'OK');
        ELSE
            RAISE e_clave_primaria;
        END IF;
        CLOSE ofertas;
    EXCEPTION
        WHEN e_clave_primaria THEN RETURN logAndResult(operacion, parametros, c_clave_primaria);
        WHEN e_datos_obligatorios THEN RETURN logAndResult(operacion, parametros, c_datos_obligatorios);
        WHEN e_cascade THEN RETURN logAndResult(operacion, parametros, c_cascade);
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
	END modOferta;

    FUNCTION bajaOferta (
        p_id oferta.id%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.bajaOferta';
        parametros logs.parametros%TYPE := 'p_id = ' || p_id;
		--
		CURSOR ofertas IS
		SELECT *
		FROM oferta
		WHERE id = p_id
		FOR UPDATE;
		oferta ofertas%ROWTYPE;
		--
		v_num NUMBER;
	BEGIN
        OPEN ofertas;
        FETCH ofertas INTO oferta;
        IF ofertas%FOUND THEN
			SELECT COUNT(1)
			INTO v_num
			FROM candidatura
			WHERE oferta_id = p_id;
			IF v_num > 0 THEN
				RAISE e_cascade;
			END IF;
			
			DELETE recibe
			WHERE oferta_id = p_id;
			
            DELETE oferta
            WHERE CURRENT OF ofertas;
			
			pq_dw.delOferta(oferta.empresa_id, oferta.plazas);
			
            RETURN logAndResult(operacion, parametros, 'OK');
        ELSE
            RAISE e_clave_primaria;
        END IF;
        CLOSE ofertas;
    EXCEPTION
        WHEN e_clave_primaria THEN RETURN logAndResult(operacion, parametros, c_clave_primaria);
        WHEN e_cascade THEN RETURN logAndResult(operacion, parametros, c_cascade);
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
	END bajaOferta;

    FUNCTION altaCandidatura (
        p_universidad_id candidatura.universidad_id%TYPE,
        p_estudiante_id candidatura.estudiante_id%TYPE,
        p_oferta_id candidatura.oferta_id%TYPE,
        p_fecha candidatura.fecha%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.altaCandidatura';
        parametros logs.parametros%TYPE := 'p_universidad_id = ' || p_universidad_id || ', p_estudiante_id = ' || p_estudiante_id || ', p_oferta_id = ' || p_oferta_id || ', p_fecha = ''' || TO_CHAR(p_fecha, 'dd/mm/rrrr') || '''';
		--
		v_num NUMBER;
	BEGIN
		SELECT COUNT(1)
		INTO v_num
		FROM cursa
		WHERE universidad_id = p_universidad_id
		 AND estudiante_id = p_estudiante_id
		 AND anyo_acad = anyoActual();
		IF v_num = 0 THEN
			RAISE e_clave_foranea;
		END IF;
		
		SELECT COUNT(1)
		INTO v_num
		FROM oferta
		WHERE id = p_oferta_id;
		IF v_num = 0 THEN
			RAISE e_clave_foranea;
		END IF;
		
		IF p_fecha IS NULL THEN
			RAISE e_datos_obligatorios;
		END IF;
		
		INSERT INTO candidatura
		VALUES (p_universidad_id, p_estudiante_id, p_oferta_id, p_fecha);
		
		pq_dw.addCandidatura();
		
		RETURN logAndResult(operacion, parametros, 'OK');
    EXCEPTION
        WHEN e_clave_foranea THEN RETURN logAndResult(operacion, parametros, c_clave_foranea);
        WHEN e_datos_obligatorios THEN RETURN logAndResult(operacion, parametros, c_datos_obligatorios);
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
	END altaCandidatura;
	
	FUNCTION modCandidatura (
        p_universidad_id candidatura.universidad_id%TYPE,
        p_estudiante_id candidatura.estudiante_id%TYPE,
        p_oferta_id candidatura.oferta_id%TYPE,
        p_fecha candidatura.fecha%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.modCandidatura';
        parametros logs.parametros%TYPE := 'p_universidad_id = ' || p_universidad_id || ', p_estudiante_id = ' || p_estudiante_id || ', p_oferta_id = ' || p_oferta_id || ', p_fecha = ''' || TO_CHAR(p_fecha, 'dd/mm/rrrr') || '''';
		--
		CURSOR candidaturas IS
		SELECT *
		FROM candidatura
		WHERE universidad_id = p_universidad_id
		 AND estudiante_id = p_estudiante_id
		 AND oferta_id = p_oferta_id
		FOR UPDATE;
		candidatura candidaturas%ROWTYPE;
	BEGIN
        OPEN candidaturas;
        FETCH candidaturas INTO candidatura;
        IF candidaturas%FOUND THEN
            IF p_fecha IS NULL THEN
                RAISE e_datos_obligatorios;
            END IF;
			
            UPDATE candidatura
            SET fecha = p_fecha
            WHERE CURRENT OF candidaturas;
			
            RETURN logAndResult(operacion, parametros, 'OK');
        ELSE
            RAISE e_clave_primaria;
        END IF;
        CLOSE candidaturas;
    EXCEPTION
        WHEN e_clave_primaria THEN RETURN logAndResult(operacion, parametros, c_clave_primaria);
        WHEN e_datos_obligatorios THEN RETURN logAndResult(operacion, parametros, c_datos_obligatorios);
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
	END modCandidatura;
	
	FUNCTION bajaCandidatura (
        p_universidad_id candidatura.universidad_id%TYPE,
        p_estudiante_id candidatura.estudiante_id%TYPE,
        p_oferta_id candidatura.oferta_id%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.bajaCandidatura';
        parametros logs.parametros%TYPE := 'p_universidad_id = ' || p_universidad_id || ', p_estudiante_id = ' || p_estudiante_id || ', p_oferta_id = ' || p_oferta_id;
		--
		CURSOR candidaturas IS
		SELECT *
		FROM candidatura
		WHERE universidad_id = p_universidad_id
		 AND estudiante_id = p_estudiante_id
		 AND oferta_id = p_oferta_id
		FOR UPDATE;
		candidatura candidaturas%ROWTYPE;
		--
		v_num NUMBER;
	BEGIN
        OPEN candidaturas;
        FETCH candidaturas INTO candidatura;
        IF candidaturas%FOUND THEN
			SELECT COUNT(1)
			INTO v_num
			FROM entrevista
			WHERE universidad_id = p_universidad_id
			 AND estudiante_id = p_estudiante_id
			 AND oferta_id = p_oferta_id;
			IF v_num > 0 THEN
				RAISE e_cascade;
			END IF;
			
            DELETE candidatura
            WHERE CURRENT OF candidaturas;
			
			pq_dw.delCandidatura();
			
            RETURN logAndResult(operacion, parametros, 'OK');
        ELSE
            RAISE e_clave_primaria;
        END IF;
        CLOSE candidaturas;
    EXCEPTION
        WHEN e_clave_primaria THEN RETURN logAndResult(operacion, parametros, c_clave_primaria);
        WHEN e_cascade THEN RETURN logAndResult(operacion, parametros, c_cascade);
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
	END bajaCandidatura;
	
    FUNCTION altaEntrevista (
        p_universidad_id entrevista.universidad_id%TYPE,
        p_estudiante_id entrevista.estudiante_id%TYPE,
        p_oferta_id entrevista.oferta_id%TYPE,
        p_empresa_id entrevista.empresa_id%TYPE,
        p_solicitante_id entrevista.solicitante_id%TYPE,
        p_fecha entrevista.fecha%TYPE,
        p_comentarios entrevista.comentarios%TYPE,
        p_es_apto entrevista.es_apto%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.altaEntrevista';
        parametros logs.parametros%TYPE := 'p_universidad_id = ' || p_universidad_id || ', p_estudiante_id = ' || p_estudiante_id || ', p_oferta_id = ' || p_oferta_id || ', p_empresa_id = ' || p_empresa_id || ', p_solicitante_id = ' || p_solicitante_id || ', p_fecha = ''' || TO_CHAR(p_fecha, 'dd/mm/rrrr') || ''', p_comentarios = ''' || p_comentarios || ''', p_es_apto = ''' || p_es_apto || '''';
		--
		v_num NUMBER;
	BEGIN
		SELECT COUNT(1)
		INTO v_num
		FROM candidatura c
		 JOIN oferta o ON c.oferta_id = o.id
		WHERE c.universidad_id = p_universidad_id
		 AND c.estudiante_id = p_estudiante_id
		 AND o.id = p_oferta_id
		 AND o.empresa_id = p_empresa_id
		 AND o.solicitante_id = p_solicitante_id;
		IF v_num = 0 THEN
			RAISE e_clave_foranea;
		END IF;
		
		IF p_fecha IS NULL OR p_es_apto IS NULL THEN
			RAISE e_datos_obligatorios;
		END IF;
		
		INSERT INTO entrevista
		VALUES (p_universidad_id, p_estudiante_id, p_oferta_id, p_empresa_id, p_solicitante_id, p_fecha, p_comentarios, p_es_apto);
		
		pq_dw.addEntrevista(p_empresa_id);
		
		RETURN logAndResult(operacion, parametros, 'OK');
    EXCEPTION
        WHEN e_clave_foranea THEN RETURN logAndResult(operacion, parametros, c_clave_foranea);
        WHEN e_datos_obligatorios THEN RETURN logAndResult(operacion, parametros, c_datos_obligatorios);
        WHEN DUP_VAL_ON_INDEX THEN RETURN logAndResult(operacion, parametros, 'ERROR ENTREVISTA DUPLICADA');
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
	END altaEntrevista;
	
	FUNCTION modEntrevista (
        p_universidad_id entrevista.universidad_id%TYPE,
        p_estudiante_id entrevista.estudiante_id%TYPE,
        p_oferta_id entrevista.oferta_id%TYPE,
        p_empresa_id entrevista.empresa_id%TYPE,
        p_solicitante_id entrevista.solicitante_id%TYPE,
        p_fecha entrevista.fecha%TYPE,
        p_comentarios entrevista.comentarios%TYPE,
        p_es_apto entrevista.es_apto%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.modEntrevista';
        parametros logs.parametros%TYPE := 'p_universidad_id = ' || p_universidad_id || ', p_estudiante_id = ' || p_estudiante_id || ', p_oferta_id = ' || p_oferta_id || ', p_empresa_id = ' || p_empresa_id || ', p_solicitante_id = ' || p_solicitante_id || ', p_fecha = ''' || TO_CHAR(p_fecha, 'dd/mm/rrrr') || ''', p_comentarios = ''' || p_comentarios || ''', p_es_apto = ''' || p_es_apto || '''';
		--
		CURSOR entrevistas IS
		SELECT *
		FROM entrevista
		WHERE universidad_id = p_universidad_id
		 AND estudiante_id = p_estudiante_id
		 AND oferta_id = p_oferta_id
		 AND empresa_id = p_empresa_id
		 AND solicitante_id = p_solicitante_id
		FOR UPDATE;
		entrevista entrevistas%ROWTYPE;
		--
		v_num NUMBER;
	BEGIN
        OPEN entrevistas;
        FETCH entrevistas INTO entrevista;
        IF entrevistas%FOUND THEN
            IF p_fecha IS NULL OR p_es_apto IS NULL THEN
                RAISE e_datos_obligatorios;
            END IF;
			
			IF p_es_apto = '0' AND entrevista.es_apto = '1' THEN
				SELECT COUNT(1)
				INTO v_num
				FROM colaboracion
				WHERE empresa_id = p_empresa_id
				 AND universidad_id = p_universidad_id
				 AND estudiante_id = p_estudiante_id
				 AND oferta_id = p_oferta_id;
				IF v_num > 0 THEN
					RAISE e_cascade;
				END IF;
			END IF;
			
            UPDATE entrevista
            SET fecha = p_fecha,
			 comentarios = p_comentarios,
			 es_apto = p_es_apto
            WHERE CURRENT OF entrevistas;
			
            RETURN logAndResult(operacion, parametros, 'OK');
        ELSE
            RAISE e_clave_primaria;
        END IF;
        CLOSE entrevistas;
    EXCEPTION
        WHEN e_clave_primaria THEN RETURN logAndResult(operacion, parametros, c_clave_primaria);
        WHEN e_datos_obligatorios THEN RETURN logAndResult(operacion, parametros, c_datos_obligatorios);
        WHEN e_cascade THEN RETURN logAndResult(operacion, parametros, c_cascade);
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
	END modEntrevista;
	
	FUNCTION bajaEntrevista (
        p_universidad_id entrevista.universidad_id%TYPE,
        p_estudiante_id entrevista.estudiante_id%TYPE,
        p_oferta_id entrevista.oferta_id%TYPE,
        p_empresa_id entrevista.empresa_id%TYPE,
        p_solicitante_id entrevista.solicitante_id%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.bajaEntrevista';
        parametros logs.parametros%TYPE := 'p_universidad_id = ' || p_universidad_id || ', p_estudiante_id = ' || p_estudiante_id || ', p_oferta_id = ' || p_oferta_id || ', p_empresa_id = ' || p_empresa_id || ', p_solicitante_id = ' || p_solicitante_id;
		--
		CURSOR entrevistas IS
		SELECT *
		FROM entrevista
		WHERE universidad_id = p_universidad_id
		 AND estudiante_id = p_estudiante_id
		 AND oferta_id = p_oferta_id
		 AND empresa_id = p_empresa_id
		 AND solicitante_id = p_solicitante_id
		FOR UPDATE;
		entrevista entrevistas%ROWTYPE;
		--
		v_num NUMBER;
	BEGIN
        OPEN entrevistas;
        FETCH entrevistas INTO entrevista;
        IF entrevistas%FOUND THEN
			SELECT COUNT(1)
			INTO v_num
			FROM colaboracion
			WHERE empresa_id = p_empresa_id
			 AND universidad_id = p_universidad_id
			 AND estudiante_id = p_estudiante_id
			 AND oferta_id = p_oferta_id;
			IF v_num > 0 THEN
				RAISE e_cascade;
			END IF;
			
            DELETE entrevista
            WHERE CURRENT OF entrevistas;
			
			pq_dw.delEntrevista(p_empresa_id);

            RETURN logAndResult(operacion, parametros, 'OK');
        ELSE
            RAISE e_clave_primaria;
        END IF;
        CLOSE entrevistas;
    EXCEPTION
        WHEN e_clave_primaria THEN RETURN logAndResult(operacion, parametros, c_clave_primaria);
        WHEN e_cascade THEN RETURN logAndResult(operacion, parametros, c_cascade);
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
	END bajaEntrevista;

    FUNCTION altaColaboracion (
        p_empresa_id colaboracion.empresa_id%TYPE,
        p_universidad_id colaboracion.universidad_id%TYPE,
        p_estudiante_id colaboracion.estudiante_id%TYPE,
        p_oferta_id colaboracion.oferta_id%TYPE,
        p_fecha_inicio colaboracion.fecha_inicio%TYPE,
        p_fecha_final colaboracion.fecha_final%TYPE,
        p_sueldo colaboracion.sueldo%TYPE,
        p_informes_esperados colaboracion.informes_esperados%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.altaColaboracion';
        parametros logs.parametros%TYPE := 'p_empresa_id = ' || p_empresa_id || ', p_universidad_id = ' || p_universidad_id || ', p_estudiante_id = ' || p_estudiante_id || ', p_oferta_id = ' || p_oferta_id || ', p_fecha_inicio = ''' || TO_CHAR(p_fecha_inicio, 'dd/mm/rrrr') || ''', p_fecha_final = ''' || TO_CHAR(p_fecha_final, 'dd/mm/rrrr') || ''', p_sueldo = ' || p_sueldo || ', p_informes_esperados = ' || p_informes_esperados;
		--
		v_num NUMBER;
		v_plazas oferta.plazas%TYPE;
		--
		v_profesor_id cursa.profesor_id%TYPE;
		--
		e_top_plazas EXCEPTION;
	BEGIN
		SELECT COUNT(1)
		INTO v_num
		FROM entrevista
		WHERE universidad_id = p_universidad_id
		 AND estudiante_id = p_estudiante_id
		 AND oferta_id = p_oferta_id
		 AND empresa_id = p_empresa_id
		 AND es_apto = '1';
		IF v_num = 0 THEN
			RAISE e_clave_foranea;
		END IF;
		
		IF p_fecha_inicio IS NULL OR p_fecha_final IS NULL OR NVL(p_informes_esperados, 0) < 1 THEN
			RAISE e_datos_obligatorios;
		END IF;
		
		IF MONTHS_BETWEEN(p_fecha_final, p_fecha_inicio) < 1 THEN
			RAISE e_duracion_baja;
		END IF;
		
		SELECT o.plazas, COUNT(c.oferta_id)
		INTO v_plazas, v_num
		FROM oferta o
		 LEFT JOIN colaboracion c ON o.id = c.oferta_id
		WHERE o.id = p_oferta_id
		GROUP BY o.plazas;
		IF v_num = v_plazas THEN
			RAISE e_top_plazas;
		END IF;
		
		SELECT profesor_id
		INTO v_profesor_id
		FROM cursa
		WHERE universidad_id = p_universidad_id
		 AND estudiante_id = p_estudiante_id
		 AND anyo_acad = anyoActual();
		IF v_profesor_id IS NULL THEN
			RAISE e_clave_foranea;
		END IF;
		
		INSERT INTO colaboracion
		VALUES (p_empresa_id, p_universidad_id, p_estudiante_id, p_oferta_id, p_fecha_inicio, p_fecha_final, p_sueldo, p_informes_esperados);
		
		pq_dw.addColaboracion(p_universidad_id, v_profesor_id, p_sueldo, MONTHS_BETWEEN(p_fecha_final, p_fecha_inicio), p_informes_esperados);
		
		RETURN logAndResult(operacion, parametros, 'OK');
    EXCEPTION
        WHEN e_clave_foranea THEN RETURN logAndResult(operacion, parametros, c_clave_foranea);
        WHEN e_datos_obligatorios THEN RETURN logAndResult(operacion, parametros, c_datos_obligatorios);
        WHEN e_duracion_baja THEN RETURN logAndResult(operacion, parametros, c_duracion_baja);
        WHEN e_top_plazas THEN RETURN logAndResult(operacion, parametros, 'ERROR NÚMERO MÁXIMO DE PLAZAS ALCANZADO');
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
	END altaColaboracion;
	
	FUNCTION modColaboracion (
        p_empresa_id colaboracion.empresa_id%TYPE,
        p_universidad_id colaboracion.universidad_id%TYPE,
        p_estudiante_id colaboracion.estudiante_id%TYPE,
        p_oferta_id colaboracion.oferta_id%TYPE,
        p_fecha_inicio colaboracion.fecha_inicio%TYPE,
        p_fecha_final colaboracion.fecha_final%TYPE,
        p_sueldo colaboracion.sueldo%TYPE,
        p_informes_esperados colaboracion.informes_esperados%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.modColaboracion';
        parametros logs.parametros%TYPE := 'p_empresa_id = ' || p_empresa_id || ', p_universidad_id = ' || p_universidad_id || ', p_estudiante_id = ' || p_estudiante_id || ', p_oferta_id = ' || p_oferta_id || ', p_fecha_inicio = ''' || TO_CHAR(p_fecha_inicio, 'dd/mm/rrrr') || ''', p_fecha_final = ''' || TO_CHAR(p_fecha_final, 'dd/mm/rrrr') || ''', p_sueldo = ' || p_sueldo || ', p_informes_esperados = ' || p_informes_esperados;
		--
		CURSOR colaboraciones IS
		SELECT *
		FROM colaboracion
		WHERE empresa_id = p_empresa_id
		 AND universidad_id = p_universidad_id
		 AND estudiante_id = p_estudiante_id
		 AND oferta_id = p_oferta_id
		FOR UPDATE;
		colaboracion colaboraciones%ROWTYPE;
		--
		v_num NUMBER;
	BEGIN
        OPEN colaboraciones;
        FETCH colaboraciones INTO colaboracion;
        IF colaboraciones%FOUND THEN
            IF p_fecha_inicio IS NULL OR p_fecha_final IS NULL OR NVL(p_informes_esperados, 0) < 1 THEN
                RAISE e_datos_obligatorios;
            END IF;
			
			IF MONTHS_BETWEEN(p_fecha_final, p_fecha_inicio) < 1 THEN
				RAISE e_duracion_baja;
			END IF;
			
			IF p_informes_esperados < colaboracion.informes_esperados THEN
				SELECT COUNT(1)
				INTO v_num
				FROM informe
				WHERE empresa_id = p_empresa_id
				 AND universidad_id = p_universidad_id
				 AND estudiante_id = p_estudiante_id
				 AND oferta_id = p_oferta_id;
				IF v_num > p_informes_esperados THEN
					RAISE e_cascade;
				END IF;
			END IF;
			
            UPDATE colaboracion
            SET fecha_inicio = p_fecha_inicio,
             fecha_final = p_fecha_final,
			 sueldo = p_sueldo,
             informes_esperados = p_informes_esperados
            WHERE CURRENT OF colaboraciones;
			
			pq_dw.updColaboracion(p_sueldo - colaboracion.sueldo, MONTHS_BETWEEN(p_fecha_final, p_fecha_inicio) - MONTHS_BETWEEN(colaboracion.fecha_final, colaboracion.fecha_inicio), p_informes_esperados - colaboracion.informes_esperados);
			
            RETURN logAndResult(operacion, parametros, 'OK');
        ELSE
            RAISE e_clave_primaria;
        END IF;
        CLOSE colaboraciones;
    EXCEPTION
        WHEN e_clave_primaria THEN RETURN logAndResult(operacion, parametros, c_clave_primaria);
        WHEN e_datos_obligatorios THEN RETURN logAndResult(operacion, parametros, c_datos_obligatorios);
        WHEN e_duracion_baja THEN RETURN logAndResult(operacion, parametros, c_duracion_baja);
        WHEN e_cascade THEN RETURN logAndResult(operacion, parametros, c_cascade);
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
	END modColaboracion;
	
	FUNCTION bajaColaboracion (
        p_empresa_id colaboracion.empresa_id%TYPE,
        p_universidad_id colaboracion.universidad_id%TYPE,
        p_estudiante_id colaboracion.estudiante_id%TYPE,
        p_oferta_id colaboracion.oferta_id%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.bajaColaboracion';
        parametros logs.parametros%TYPE := 'p_empresa_id = ' || p_empresa_id || ', p_universidad_id = ' || p_universidad_id || ', p_estudiante_id = ' || p_estudiante_id || ', p_oferta_id = ' || p_oferta_id;
		--
		CURSOR colaboraciones IS
		SELECT *
		FROM colaboracion
		WHERE empresa_id = p_empresa_id
		 AND universidad_id = p_universidad_id
		 AND estudiante_id = p_estudiante_id
		 AND oferta_id = p_oferta_id
		FOR UPDATE;
		colaboracion colaboraciones%ROWTYPE;
		--
		v_num NUMBER;
		--
		v_profesor_id cursa.profesor_id%TYPE;
	BEGIN
        OPEN colaboraciones;
        FETCH colaboraciones INTO colaboracion;
        IF colaboraciones%FOUND THEN
			SELECT COUNT(1)
			INTO v_num
			FROM informe
			WHERE empresa_id = p_empresa_id
			 AND universidad_id = p_universidad_id
			 AND estudiante_id = p_estudiante_id
			 AND oferta_id = p_oferta_id;
			IF v_num > 0 THEN
				RAISE e_cascade;
			END IF;
			
            DELETE colaboracion
            WHERE CURRENT OF colaboraciones;
			
			SELECT profesor_id
			INTO v_profesor_id
			FROM cursa
			WHERE universidad_id = p_universidad_id
			 AND estudiante_id = p_estudiante_id
			 AND anyo_acad = anyoActual();
			
			pq_dw.delColaboracion(colaboracion.universidad_id, v_profesor_id, colaboracion.sueldo, MONTHS_BETWEEN(colaboracion.fecha_final, colaboracion.fecha_inicio), colaboracion.informes_esperados);
			
            RETURN logAndResult(operacion, parametros, 'OK');
        ELSE
            RAISE e_clave_primaria;
        END IF;
        CLOSE colaboraciones;
    EXCEPTION
        WHEN e_clave_primaria THEN RETURN logAndResult(operacion, parametros, c_clave_primaria);
        WHEN e_cascade THEN RETURN logAndResult(operacion, parametros, c_cascade);
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
	END bajaColaboracion;
	
	FUNCTION altaInspeccion (
        p_empresa_id inspeccion.empresa_id%TYPE,
        p_fecha inspeccion.fecha%TYPE,
        p_muestreos inspeccion.muestreos%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.altaInspeccion';
        parametros logs.parametros%TYPE := 'p_empresa_id = ' || p_empresa_id || ', p_fecha = ''' || TO_CHAR(p_fecha, 'dd/mm/rrrr') || ''', p_muestreos = ''' || p_muestreos || '''';
		--
		v_num NUMBER;
	BEGIN
		SELECT COUNT(1)
		INTO v_num
		FROM empresa
		WHERE id = p_empresa_id;
		IF v_num = 0 THEN
			RAISE e_clave_foranea;
		END IF;
		
		IF p_fecha IS NULL OR p_muestreos IS NULL THEN
			RAISE e_datos_obligatorios;
		END IF;
		
		INSERT INTO inspeccion
		VALUES (NULL, p_empresa_id, p_fecha, p_muestreos);
		
		pq_dw.addInspeccion(p_empresa_id);
		
		RETURN logAndResult(operacion, parametros, 'OK');
    EXCEPTION
        WHEN e_clave_foranea THEN RETURN logAndResult(operacion, parametros, c_clave_foranea);
        WHEN e_datos_obligatorios THEN RETURN logAndResult(operacion, parametros, c_datos_obligatorios);
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
	END altaInspeccion;

    FUNCTION modInspeccion (
        p_id inspeccion.id%TYPE,
        p_fecha inspeccion.fecha%TYPE,
        p_muestreos inspeccion.muestreos%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.modInspeccion';
        parametros logs.parametros%TYPE := 'p_id = ' || p_id || ', p_fecha = ''' || TO_CHAR(p_fecha, 'dd/mm/rrrr') || ''', p_muestreos = ''' || p_muestreos || '''';
		--
		CURSOR inspecciones IS
		SELECT *
		FROM inspeccion
		WHERE id = p_id
		FOR UPDATE;
		inspeccion inspecciones%ROWTYPE;
	BEGIN
        OPEN inspecciones;
        FETCH inspecciones INTO inspeccion;
        IF inspecciones%FOUND THEN
            IF p_fecha IS NULL OR p_muestreos IS NULL THEN
                RAISE e_datos_obligatorios;
            END IF;
			
            UPDATE inspeccion
            SET fecha = p_fecha,
             muestreos = p_muestreos
            WHERE CURRENT OF inspecciones;
						
            RETURN logAndResult(operacion, parametros, 'OK');
        ELSE
            RAISE e_clave_primaria;
        END IF;
        CLOSE inspecciones;
    EXCEPTION
        WHEN e_clave_primaria THEN RETURN logAndResult(operacion, parametros, c_clave_primaria);
        WHEN e_datos_obligatorios THEN RETURN logAndResult(operacion, parametros, c_datos_obligatorios);
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
	END modInspeccion;

    FUNCTION bajaInspeccion (
        p_id inspeccion.id%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.bajaInspeccion';
        parametros logs.parametros%TYPE := 'p_id = ' || p_id;
		--
		CURSOR inspecciones IS
		SELECT *
		FROM inspeccion
		WHERE id = p_id
		FOR UPDATE;
		inspeccion inspecciones%ROWTYPE;
	BEGIN
        OPEN inspecciones;
        FETCH inspecciones INTO inspeccion;
        IF inspecciones%FOUND THEN
            DELETE inspeccion
            WHERE CURRENT OF inspecciones;
			
			pq_dw.delInspeccion(inspeccion.empresa_id);
						
            RETURN logAndResult(operacion, parametros, 'OK');
        ELSE
            RAISE e_clave_primaria;
        END IF;
        CLOSE inspecciones;
    EXCEPTION
        WHEN e_clave_primaria THEN RETURN logAndResult(operacion, parametros, c_clave_primaria);
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
	END bajaInspeccion;
	
	FUNCTION altaInforme (
        p_empresa_id informe.empresa_id%TYPE,
        p_universidad_id informe.universidad_id%TYPE,
        p_estudiante_id informe.estudiante_id%TYPE,
        p_oferta_id informe.oferta_id%TYPE,
        p_solicitante_id informe.solicitante_id%TYPE,
        p_fecha informe.fecha%TYPE,
        p_comentarios informe.comentarios%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.altaInforme';
        parametros logs.parametros%TYPE := 'p_empresa_id = ' || p_empresa_id || ', p_universidad_id = ' || p_universidad_id || ', p_estudiante_id = ' || p_estudiante_id || ', p_oferta_id = ' || p_oferta_id || ', p_solicitante_id = ' || p_solicitante_id || ', p_fecha = ''' || TO_CHAR(p_fecha, 'dd/mm/rrrr') || ''', p_comentarios = ''' || p_comentarios || '''';
		--
		v_num NUMBER;
	BEGIN
		SELECT COUNT(1)
		INTO v_num
		FROM colaboracion c
		 JOIN oferta o ON c.oferta_id = o.id
		WHERE c.empresa_id = p_empresa_id
		 AND c.universidad_id = p_universidad_id
		 AND c.estudiante_id = p_estudiante_id
		 AND c.oferta_id = p_oferta_id
		 AND o.solicitante_id = p_solicitante_id;
		IF v_num = 0 THEN
			RAISE e_clave_foranea;
		END IF;
		
		IF p_comentarios IS NULL THEN
			RAISE e_datos_obligatorios;
		END IF;
		
		INSERT INTO informe
		VALUES (p_empresa_id, p_universidad_id, p_estudiante_id, p_oferta_id, p_solicitante_id, p_fecha, p_comentarios);
		
		pq_dw.addInforme();
		
		RETURN logAndResult(operacion, parametros, 'OK');
    EXCEPTION
        WHEN e_clave_foranea THEN RETURN logAndResult(operacion, parametros, c_clave_foranea);
        WHEN e_datos_obligatorios THEN RETURN logAndResult(operacion, parametros, c_datos_obligatorios);
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
	END altaInforme;

    FUNCTION modInforme (
        p_empresa_id informe.empresa_id%TYPE,
        p_universidad_id informe.universidad_id%TYPE,
        p_estudiante_id informe.estudiante_id%TYPE,
        p_oferta_id informe.oferta_id%TYPE,
        p_solicitante_id informe.solicitante_id%TYPE,
        p_fecha informe.fecha%TYPE,
        p_comentarios informe.comentarios%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.modInforme';
        parametros logs.parametros%TYPE := 'p_empresa_id = ' || p_empresa_id || ', p_universidad_id = ' || p_universidad_id || ', p_estudiante_id = ' || p_estudiante_id || ', p_oferta_id = ' || p_oferta_id || ', p_solicitante_id = ' || p_solicitante_id || ', p_fecha = ''' || TO_CHAR(p_fecha, 'dd/mm/rrrr') || ''', p_comentarios = ''' || p_comentarios || '''';
		--
		CURSOR informes IS
		SELECT *
		FROM informe
		WHERE empresa_id = p_empresa_id
		 AND universidad_id = p_universidad_id
		 AND estudiante_id = p_estudiante_id
		 AND oferta_id = p_oferta_id
		 AND solicitante_id = p_solicitante_id
		 AND fecha = p_fecha
		FOR UPDATE;
		informe informes%ROWTYPE;
	BEGIN
        OPEN informes;
        FETCH informes INTO informe;
        IF informes%FOUND THEN
            IF p_comentarios IS NULL THEN
                RAISE e_datos_obligatorios;
            END IF;
			
            UPDATE informe
            SET comentarios = p_comentarios
            WHERE CURRENT OF informes;
						
            RETURN logAndResult(operacion, parametros, 'OK');
        ELSE
            RAISE e_clave_primaria;
        END IF;
        CLOSE informes;
    EXCEPTION
        WHEN e_clave_primaria THEN RETURN logAndResult(operacion, parametros, c_clave_primaria);
        WHEN e_datos_obligatorios THEN RETURN logAndResult(operacion, parametros, c_datos_obligatorios);
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
	END modInforme;

    FUNCTION bajaInforme (
        p_empresa_id informe.empresa_id%TYPE,
        p_universidad_id informe.universidad_id%TYPE,
        p_estudiante_id informe.estudiante_id%TYPE,
        p_oferta_id informe.oferta_id%TYPE,
        p_solicitante_id informe.solicitante_id%TYPE,
        p_fecha informe.fecha%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.bajaInforme';
        parametros logs.parametros%TYPE := 'p_empresa_id = ' || p_empresa_id || ', p_universidad_id = ' || p_universidad_id || ', p_estudiante_id = ' || p_estudiante_id || ', p_oferta_id = ' || p_oferta_id || ', p_solicitante_id = ' || p_solicitante_id || ', p_fecha = ''' || TO_CHAR(p_fecha, 'dd/mm/rrrr') || '''';
		--
		CURSOR informes IS
		SELECT *
		FROM informe
		WHERE empresa_id = p_empresa_id
		 AND universidad_id = p_universidad_id
		 AND estudiante_id = p_estudiante_id
		 AND oferta_id = p_oferta_id
		 AND solicitante_id = p_solicitante_id
		 AND fecha = p_fecha
		FOR UPDATE;
		informe informes%ROWTYPE;
		--
		v_num NUMBER;
	BEGIN
        OPEN informes;
        FETCH informes INTO informe;
        IF informes%FOUND THEN
            SELECT COUNT(1)
			INTO v_num
			FROM informe_final
			WHERE empresa_id = p_empresa_id
			 AND universidad_id = p_universidad_id
			 AND estudiante_id = p_estudiante_id
			 AND oferta_id = p_oferta_id
			 AND solicitante_id = p_solicitante_id
			 AND fecha = p_fecha;
			IF v_num > 0 THEN
				RAISE e_cascade;
			END IF;
			
            DELETE informe
            WHERE CURRENT OF informes;
			
			pq_dw.delInforme();
						
            RETURN logAndResult(operacion, parametros, 'OK');
        ELSE
            RAISE e_clave_primaria;
        END IF;
        CLOSE informes;
    EXCEPTION
        WHEN e_clave_primaria THEN RETURN logAndResult(operacion, parametros, c_clave_primaria);
        WHEN e_cascade THEN RETURN logAndResult(operacion, parametros, c_cascade);
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
	END bajaInforme;

    FUNCTION altaInformeFinal (
        p_empresa_id informe_final.empresa_id%TYPE,
        p_universidad_id informe_final.universidad_id%TYPE,
        p_estudiante_id informe_final.estudiante_id%TYPE,
        p_oferta_id informe_final.oferta_id%TYPE,
        p_solicitante_id informe_final.solicitante_id%TYPE,
        p_fecha informe_final.fecha%TYPE,
        p_comentarios informe.comentarios%TYPE,
        p_valoracion informe_final.valoracion%TYPE,
        p_es_apto informe_final.es_apto%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.altaInformeFinal';
        parametros logs.parametros%TYPE := 'p_empresa_id = ' || p_empresa_id || ', p_universidad_id = ' || p_universidad_id || ', p_estudiante_id = ' || p_estudiante_id || ', p_oferta_id = ' || p_oferta_id || ', p_solicitante_id = ' || p_solicitante_id || ', p_fecha = ''' || TO_CHAR(p_fecha, 'dd/mm/rrrr') || ''', p_comentarios = ''' || p_comentarios || ''', p_valoracion = ''' || p_valoracion || ''', p_es_apto = ''' || p_es_apto || '''';
		--
		v_resultado logs.resultado%TYPE;
		e_resultado EXCEPTION;
		--
		v_titulacion_id cursa.titulacion_id%TYPE;
	BEGIN
		IF p_valoracion IS NULL OR p_es_apto IS NULL THEN
			RAISE e_datos_obligatorios;
		END IF;

		v_resultado := altaInforme(p_empresa_id, p_universidad_id, p_estudiante_id, p_oferta_id, p_solicitante_id, p_fecha, p_comentarios);
		IF v_resultado <> 'OK' THEN
			RAISE e_resultado;
		END IF;
		
		INSERT INTO informe_final
		VALUES (p_empresa_id, p_universidad_id, p_estudiante_id, p_oferta_id, p_solicitante_id, p_fecha, p_valoracion, p_es_apto);
		
		SELECT titulacion_id
		INTO v_titulacion_id
		FROM cursa
		WHERE universidad_id = p_universidad_id
		 AND estudiante_id = p_estudiante_id
		 AND anyo_acad = anyoActual();
		
		pq_dw.addInformeFinal(p_universidad_id, v_titulacion_id, p_es_apto);
		
		RETURN logAndResult(operacion, parametros, 'OK');
    EXCEPTION
        WHEN e_datos_obligatorios THEN RETURN logAndResult(operacion, parametros, c_datos_obligatorios);
        WHEN e_resultado THEN RETURN logAndResult(operacion, parametros, v_resultado);
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
	END altaInformeFinal;

    FUNCTION modInformeFinal (
        p_empresa_id informe_final.empresa_id%TYPE,
        p_universidad_id informe_final.universidad_id%TYPE,
        p_estudiante_id informe_final.estudiante_id%TYPE,
        p_oferta_id informe_final.oferta_id%TYPE,
        p_solicitante_id informe_final.solicitante_id%TYPE,
        p_fecha informe_final.fecha%TYPE,
        p_comentarios informe.comentarios%TYPE,
        p_valoracion informe_final.valoracion%TYPE,
        p_es_apto informe_final.es_apto%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.modInformeFinal';
        parametros logs.parametros%TYPE := 'p_empresa_id = ' || p_empresa_id || ', p_universidad_id = ' || p_universidad_id || ', p_estudiante_id = ' || p_estudiante_id || ', p_oferta_id = ' || p_oferta_id || ', p_solicitante_id = ' || p_solicitante_id || ', p_fecha = ''' || TO_CHAR(p_fecha, 'dd/mm/rrrr') || ''', p_comentarios = ''' || p_comentarios || ''', p_valoracion = ''' || p_valoracion || ''', p_es_apto = ''' || p_es_apto || '''';
		--
		CURSOR informes IS
		SELECT *
		FROM informe i
		 JOIN informe_final f ON (
			i.empresa_id = f.empresa_id
			AND i.universidad_id = f.universidad_id
			AND i.estudiante_id = f.estudiante_id
			AND i.oferta_id = f.oferta_id
			AND i.solicitante_id = f.solicitante_id
			AND i.fecha = f.fecha
		 )
		WHERE i.empresa_id = p_empresa_id
		 AND i.universidad_id = p_universidad_id
		 AND i.estudiante_id = p_estudiante_id
		 AND i.oferta_id = p_oferta_id
		 AND i.solicitante_id = p_solicitante_id
		 AND i.fecha = p_fecha
		FOR UPDATE OF f.valoracion, f.es_apto;
		informe informes%ROWTYPE;
		--
		v_resultado logs.resultado%TYPE;
		e_resultado EXCEPTION;
		--
		v_titulacion_id cursa.titulacion_id%TYPE;
	BEGIN
		OPEN informes;
		FETCH informes INTO informe;
		IF informes%FOUND THEN
			IF p_comentarios IS NULL OR p_valoracion IS NULL OR p_es_apto IS NULL THEN
				RAISE e_datos_obligatorios;
			END IF;
			
			IF p_comentarios <> informe.comentarios THEN
				v_resultado := modInforme(p_empresa_id, p_universidad_id, p_estudiante_id, p_oferta_id, p_solicitante_id, p_fecha, p_comentarios);
				
				IF v_resultado <> 'OK' THEN
					RAISE e_resultado;
				END IF;
			END IF;
			
			UPDATE informe_final
			SET valoracion = p_valoracion,
			 es_apto = p_es_apto
			WHERE CURRENT OF informes;
			
			IF p_es_apto <> informe.es_apto THEN
				SELECT titulacion_id
				INTO v_titulacion_id
				FROM cursa
				WHERE universidad_id = p_universidad_id
				 AND estudiante_id = p_estudiante_id
				 AND anyo_acad = anyoActual();

				pq_dw.updInformeFinal(p_universidad_id, v_titulacion_id, p_es_apto);
			END IF;
		ELSE
			RAISE e_clave_primaria;
		END IF;
		CLOSE informes;

		RETURN logAndResult(operacion, parametros, 'OK');
	EXCEPTION
		WHEN e_clave_primaria THEN RETURN logAndResult(operacion, parametros, c_clave_primaria);
        WHEN e_datos_obligatorios THEN RETURN logAndResult(operacion, parametros, c_datos_obligatorios);
        WHEN e_resultado THEN RETURN logAndResult(operacion, parametros, v_resultado);
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
	END modInformeFinal;

    FUNCTION bajaInformeFinal (
        p_empresa_id informe_final.empresa_id%TYPE,
        p_universidad_id informe_final.universidad_id%TYPE,
        p_estudiante_id informe_final.estudiante_id%TYPE,
        p_oferta_id informe_final.oferta_id%TYPE,
        p_solicitante_id informe_final.solicitante_id%TYPE,
        p_fecha informe_final.fecha%TYPE
    ) RETURN logs.resultado%TYPE IS
        operacion logs.operacion%TYPE := 'pq_abm.modInformeFinal';
        parametros logs.parametros%TYPE := 'p_empresa_id = ' || p_empresa_id || ', p_universidad_id = ' || p_universidad_id || ', p_estudiante_id = ' || p_estudiante_id || ', p_oferta_id = ' || p_oferta_id || ', p_solicitante_id = ' || p_solicitante_id || ', p_fecha = ''' || TO_CHAR(p_fecha, 'dd/mm/rrrr') || '''';
		--
		CURSOR informes IS
		SELECT *
		FROM informe_final
		WHERE empresa_id = p_empresa_id
		 AND universidad_id = p_universidad_id
		 AND estudiante_id = p_estudiante_id
		 AND oferta_id = p_oferta_id
		 AND solicitante_id = p_solicitante_id
		 AND fecha = p_fecha
		FOR UPDATE;
		informe informes%ROWTYPE;
		--
		v_resultado logs.resultado%TYPE;
		e_resultado EXCEPTION;
		--
		v_titulacion_id cursa.titulacion_id%TYPE;
	BEGIN
		OPEN informes;
		FETCH informes INTO informe;
		IF informes%FOUND THEN
			DELETE informe_final
			WHERE CURRENT OF informes;
			
			v_resultado := bajaInforme(p_empresa_id, p_universidad_id, p_estudiante_id, p_oferta_id, p_solicitante_id, p_fecha);
			IF v_resultado <> 'OK' THEN
				RAISE e_resultado;
			END IF;
			
			SELECT titulacion_id
			INTO v_titulacion_id
			FROM cursa
			WHERE universidad_id = p_universidad_id
			 AND estudiante_id = p_estudiante_id
			 AND anyo_acad = anyoActual();

			pq_dw.delInformeFinal(p_universidad_id, v_titulacion_id, informe.es_apto);
		ELSE
			RAISE e_clave_primaria;
		END IF;
		CLOSE informes;

		RETURN logAndResult(operacion, parametros, 'OK');
	EXCEPTION
		WHEN e_clave_primaria THEN RETURN logAndResult(operacion, parametros, c_clave_primaria);
        WHEN e_resultado THEN RETURN logAndResult(operacion, parametros, v_resultado);
        WHEN OTHERS THEN RETURN logAndResult(operacion, parametros, 'ERROR ' || SUBSTR(SQLERRM, 1, 950));
	END bajaInformeFinal;

END pq_abm;
